;
; lowlevel.asm - ATA lowlevel routines
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------------------
;Default waiting timeout
;---------------------------------------
defaultwait	.proc
		ldx #1			;~8 sec
		ldy #0
		rts
		.pend
                
OP_read2048_here .proc
		jsr OP_read512_here
                jsr OP_read512
                jsr OP_read512
                #jmps OP_read512
;---------------------------------------
;Read 512 bytes to ram_exp_buffer0
;---------------------------------------
OP_read512_here	.proc
		jsr init_pom1_y
OP_read512	.proc
		jsr read_buffer
;--------------------------------------
;Read 2-256 bytes from buffer to (pom1) return C=0 if end of buffer
;---------------------------------------
read_buffer	.proc
		lda data_g

		lda low_reg		;0-7 BIT
		#sta_zp_y pom1
		iny

		lda hig_reg		;8-15 BIT
		#sta_zp_y pom1
		iny

		bne read_buffer
		inc pom2
		rts
		.pend
;--------------------------------------
;Read 512 bytes to buffer
;---------------------------------------
read_buffer_cfs	.proc
		jsr rwpn
lp		lda data_g

		lda low_reg		;0-7 BIT
		#sta_zp_y pom1

		lda hig_reg		;8-15 BIT
		#sta_zp_y pom4

		lda data_g

		lda low_reg		;0-7 BIT
		#sta_zp_y pom7

		lda hig_reg		;8-15 BIT
		#sta_zp_y pom10

		iny
		bpl lp
		rts
		.pend

rwpn		.proc
		.if SUPERCPU=1
		ldy #1
		sty pom3
		sty pom6
		sty pom9
		sty pom12
		dey
		.else
		ldy #0
		.fi
		sty pom1
		sty pom7
		lda #$80
		sta pom4
		sta pom10
		stx pom2
		stx pom5
		inx
		stx pom8
		stx pom11
		rts
		.pend
;--------------------------------------
;Write 512 bytes from buffer
;---------------------------------------
write_buffer_cfs .proc
		jsr rwpn
lp		#lda_zp_y pom1
		sta low_reg		;0-7 BIT

		#lda_zp_y pom4
		sta hig_reg		;8-15 BIT

		sta data_g

		#lda_zp_y pom7
		sta low_reg		;0-7 BIT

		#lda_zp_y pom10
		sta hig_reg		;8-15 BIT

		sta data_g

		iny
		bpl lp
		rts
		.pend
;--------------------------------------
;Get type of currently selected drive
;---------------------------------------
OP_get_current_drive_type .proc
		#ldatax current_drive_exp
		lda drive_type_exp,x
		rts
		.pend

atapi_rw	.proc
		jsr OP_atapi_packet	;preserve Y!
		tya
		#pw 8
		lda sector_g
		lda low_reg
		#pw 5
		lda cyllow_g
		lda low_reg
		#pw 4
		lda cylhi_g
		lda low_reg
		#pw 3
		lda head_g
		lda low_reg
		and #15
		#pw 2
		rts
		.pend
;---------------------------------------
;Start reading Y sectors, head is in A
;---------------------------------------
OP_read		.proc
		jsr OP_get_current_drive_type
		bpl ata
		lda #ATAPI_CMD_READ_10
		jsr atapi_rw
		jmp OP_atapi_datain_command

ata		sty low_reg
		sty nsector_g
		lda #read_d
		gne ata_data_command
		.pend
;---------------------------------------
;Start writeing Y sectors, head is in A
;---------------------------------------
OP_write	.proc
		jsr OP_get_current_drive_type
		bpl ata
		lda #ATAPI_CMD_WRITE_10
		jsr atapi_rw
		jmp OP_atapi_dataout_command

ata		sty low_reg
		sty nsector_g
		lda #write_d
		gne ata_data_command
		.pend
;--------------------------------------
;Select current drive, write A to HEAD reg, exit with C=1 if timeout
;---------------------------------------
head_select_drive .proc
		tay
		jsr OP_get_current_drive_type
		bmi atapi_select_drive
;--------------------------------------
;Select current ATA drive, write Y to HEAD reg and exit with C=1 if timeout
;---------------------------------------
ata_select_drive .proc
		tya
ata_select_drive_A .proc
		jsr OP_select_common
;--------------------------------------
;Wait BSY=0 & DRDY=1, and exit with C=1 if timeout
;---------------------------------------
tstbsydrdy1	.proc
		jsr defaultwait
lp		jsr BACK_with_C1_on_timeout_or_C0
		bit status_g
		bit low_reg
		bmi lp			;Wait BSY==0 &&
		bvc lp			;     DRDY==1
		rts
		.pend
;--------------------------------------
;Select current ATAPI drive, write Y to HEAD reg, and exit with C=1 if timeout
;---------------------------------------
atapi_select_drive .proc
		tya
atapi_select_drive_A .proc
		jsr OP_select_common
;--------------------------------------
;Wait BSY=0 & DRQ=0, and exit with C=1 if timeout
;---------------------------------------
tstbsydrq0	.proc
		jsr defaultwait
		lda #%10001000		;Wait BSY==0 && DRQ==0
lp		jsr BACK_with_C1_on_timeout_or_C0
		bit status_g
		bit low_reg
		bne lp
		rts
		.pend
;--------------------------------------
;Write A to HEAD reg with current master/slave.
;---------------------------------------
OP_select_common .proc
		and #%01001111
		pha
		lda current_drive_exp
		lsr			;master/slave?
                .if DRIVES=4
                bne *+5
                sta select0_g
                beq *+5
                sta select1_g
                .fi
		pla
		bcc *+4
		ora #%00010000
		sta low_reg
		sta head_g
		rts
		.pend
;--------------------------------------
;Wait busy
;---------------------------------------
wait_busy	.proc
		jsr OP_get_current_drive_type
		bpl OP_ata_waitme
		gmi OP_atapi_waitme
;--------------------------------------
;ATA PIO NON-DATA command
;Exit with C=1 on timeout/error
;---------------------------------------
ata_command	.proc
		sta low_reg
		sta comm_g
OP_ata_waitme	.proc
		jsr defaultwait
		bit astat_g		;Read alternate status and ignore results
lp		jsr BACK_with_C1_on_timeout_or_C0
		bit status_g
		lda low_reg
		bmi lp			;Wait BSY==0
		lsr			;ERR?
		lda #EATAERROR
x		rts
		.pend
;--------------------------------------
;ATA PIO DATA command
;Exit with C=1 on timeout/error
;---------------------------------------
ata_data_command .proc
		jsr ata_command
		bcs ata_pi_nomore_data.rt
;--------------------------------------
;Check if there's (no)more data
;Exit with C=1 on error
;---------------------------------------
ata_pi_more_data .proc
		lda #%00001000		;DRQ==0?
                .byte SKIPLDA0
ata_pi_nomore_data .proc
		#lda0			;DRQ==1?
ke		bit status_g
		eor low_reg
		and #%00001000
		cmp #%00001000
		lda #EUNEXP
rt		rts
		.pend
;---------------------------------------
;Return current_drive_exp*4+3 in X and 3 in Y
;---------------------------------------
OP_get_current_drv_XY .proc
		ldy #3
OP_get_current_drv_X .proc
		lda current_drive_exp
		asl
		asl
		ora #3
		tax
		rts
		.pend
;---------------------------------------
;init Y to 0 and pom1 pom2 pom3
;---------------------------------------
init_pom1_y	.proc
		.if SUPERCPU=1
		ldy #1
		sty pom3		;SCPU Page 1
		dey
		sty pom1
		.else
		ldy #0
		sty pom1
		.fi
		sta pom2
		rts
		.pend
;---------------------------------------
;Exit caller with C=1 on timeout, otherwise C=0
;---------------------------------------
BACK_with_C1_on_timeout_or_C0 .proc
		clc
		dex
		bne wait_a_bit
		dey
		bne ok
		pla
		pla
		sec
		lda #ETIMEOUT
ok		rts
		.pend
;---------------------------------------
;Wait a small time (more than 48ms)
;---------------------------------------
wait_50ms	.proc
		#phx_a
		ldx #0
mu		jsr wait_a_bit
		inx
		bne mu
		#plx_a
		rts
		.pend
;---------------------------------------
;Wait a small time
;---------------------------------------
wait_a_bit	.proc
		jsr *+3
		jsr *+3
		pha
		lda $d012
		eor $d012		;works everywhere, on scpu too
		beq *-3
		pla
		rts
		.pend
;---------------------------------------
;Check if next_sector is null
;---------------------------------------
next_sector0	.proc
                lda next_sector
                and #$4f
                ora next_sector+1
                ora next_sector+2
                ora next_sector+3
                rts
		.pend
