;
; mon.asm - Monitor
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

		.comment
a_acc		= 0			;A		6502		;fix 0
a_imp		= 1			;				;?
a_imm		= 2			;#$FF				;?
a_zpa		= 3			;$FF				;?
a_zpx		= 4			;$FF,x				;?
a_zpy		= 5			;$FF,y				;?
a_zxi		= 6			;($FF,x)			;fix 1
a_ziy		= 7			;($FF),y			;fix 1
a_abx		= 8			;$FFFF,x			;?
a_aby		= 9			;$FFFF,y			;?
a_abs		= 10			;$FFFF				;?
a_ind		= 11			;($FFFF)			;fix 0
a_rel		= 12			;$FFFF				;fix 0

a_lng   	= 13			;$FFFFFF		65816	;?
a_lnx		= 14    		;$FFFFFF,x                      ;fix 1
a_zsy		= 15    		;($FF,s),y			;fix 1
a_zps		= 16    		;$FF,s				;fix 1
a_zpi		= 17    		;($FF)				;fix 0
a_zpI		= 18			;[$FF]				;fix 1
a_zIy		= 19			;[$FF],y			;fix 1
a_xya   	= 20			;$FF,$FF			;fix 0
a_rll   	= 21			;$FFFF				;fix 0
a_axi   	= 22			;($FFFF,x)			;fix 0
		.endc
a_acc		= 0			;A		    		;fix 0
a_zxi		= 0			;($FF,x)			;fix 1
a_ind		= 1 			;($FFFF)			;fix 0
a_ziy		= 1     		;($FF),y			;fix 1
a_axi   	= 2 			;($FFFF,x)			;fix 0
a_lnx		= 2     		;$FFFFFF,x                      ;fix 1
a_zpi		= 3     		;($FF)				;fix 0
a_zsy		= 3     		;($FF,s),y			;fix 1
a_xya   	= 4 			;$FF,$FF			;fix 0
a_zpI		= 4 			;[$FF]				;fix 1
a_rll   	= 5 			;$FFFF				;fix 0
a_zIy		= 5 			;[$FF],y			;fix 1
a_rel		= 6 			;$FFFF				;fix 0
a_zps		= 6    			;$FF,s				;fix 1
a_imp		= 7			;				;?
a_imm		= 8			;#$FF				;?
a_zpa		= 9			;$FF				;?
a_zpx		= 10			;$FF,x				;?
a_zpy		= 11			;$FF,y				;?
a_abx		= 12    		;$FFFF,x			;?
a_aby		= 13    		;$FFFF,y			;?
a_abs		= 14			;$FFFF				;?
a_lng   	= 15			;$FFFFFF	             	;?

m_p1		= $fb
m_p2		= $fd
m_p3		= $13			;3 byte!
m_p4		= $22
m_pos		= $2
m_tmp		= $7
m_tmp2		= $8
m_tmp3		= $9			;word
m_prefetch	= $b
m_01		= $c
mon_setprefetch	= $16
mon_getbyte	= $19
mon_storebyte	= $1c
mon_exec	= $1f
m_ascii		= $57
m_inv		= $c7
m_tsp		= $5f
m_tstack	= $60

monitor_buffer	= $200

monitor_start	.proc
		ldx #5
lf		pla
		sta monitor_freeze_exp,x
		dex
		bpl lf
		.if SUPERCPU=1
		tsc
		sta monitor_sp_exp
		.else
		tsx
		stx monitor_sp_exp
		.fi

		ldx #2
cpu		lda $00,x
		sta monitor_data_exp,x
		inx
		bne cpu
		ldx #63
cpu3		lda $d000,x
		sta monitor_vic_exp,x
		cpx #32
		bcs vy
		lda $d400,x
		sta monitor_sid_exp,x
		cpx #16
		bcs vy
		lda $dc00,x
		sta monitor_cia1_exp,x
		lda $dd00,x
		sta monitor_cia2_exp,x
vy		dex
		bpl cpu3

		lda #$10
		sta $dc0e
		sta $dc0f
		sta $dd0e
		sta $dd0f

		ldx #3
iyfd		lda $dc04,x
		sta monitor_cia1_exp+4,x
		lda $dd04,x
		sta monitor_cia2_exp+4,x
		dex
		bpl iyfd
                #lda0
		sta m_tsp
		sta monitor_tod_exp
		sta $d01a
		lda #$fc
		sta $d030		;C128

		lda monitor_vic_exp+$11
		and #$7f
		sta monitor_vic_exp+$11

		jsr mon_waitretrace

		lda #$ff
		sta $d019
		lda #1
		sta $d01a

dpu2		lda $d019
		bpl dpu1
		ldx $d012
		lda $d011
		bit $d011
		bpl *-3
		gmi vok2
dpu1		bit $d011
		bpl dpu2

dpu4		lda $d019
		bpl dpu3
		ldx $d012
		lda $d011
vok2		and #$80
		ora monitor_vic_exp+$11
		sta monitor_vic_exp+$11
		#stx_l_a monitor_vic_exp+$12
		lda $d011
		bmi *-3
		gpl vok
dpu3		lda $d011
		bmi dpu4
		sta $d011		;impossible interrupt
		lda #$80		;but $d012 is still interresting
		ora monitor_vic_exp+$11
		sta monitor_vic_exp+$11
		gmi dpu2
vok
		lda #11
		sta $d011
		lda setup_moncolors_exp
		sta $d020
                
                .if SUPERCPU=1
                lda scpu_optimv2
                sta monitor_d0b3_exp
                sta scpu_hwenable
                sta scpu_nooptim
                sta scpu_hwdisable
                .else
		lda monitor_data_exp
		eor #7
		ora monitor_data_exp+1
		and #7
		sta monitor_01_exp
                .fi

		ldy #1
		sty m_p1+1
		.if SUPERCPU
		sty m_p2+2
		.fi
		dey
		sty m_p1
		sty m_p2
		lda #>(monitor_data_exp+$100)
		sta m_p2+1
		ldx #6
vg		lda (m_p1),y
		#sta_zp_y m_p2
		iny
		bne vg
		inc m_p1+1
		inc m_p2+1
		dex
		bpl vg

		sty $dd03
		stx $dd02
		lda $dd00
		sta monitor_cia2_exp
		sty $dd02
		stx $dd03
		lda $dd01
		sta monitor_cia2_exp+1
		sty $dd03

		sty $dc03
		stx $dc02
		lda $dc00
		sta monitor_cia1_exp
		sty $dc02
		stx $dc03
		lda $dc01
		sta monitor_cia1_exp+1
		sty $dc03

		inx
cp		lda $d800,x
		asl
		asl
		asl
		asl
		sta m_pos
		lda $d900,x
		and #15
		ora m_pos
		sta monitor_d800_exp,x
		lda $da00,x
		asl
		asl
		asl
		asl
		sta m_pos
		lda $db00,x
		and #15
		ora m_pos
		sta monitor_d800_exp+$100,x
		inx
		bne cp

		inx
		stx $dc04
		stx $dc06
		stx $dd04
		stx $dd06
		dex
		stx $dc05
		stx $dc07
		stx $dd05
		stx $dd07

		ldx #9
		stx $dc0e
		#lda0
		bit $dc0d
		bpl *+4
		ora #%10000001
		stx $dc0f
		bit $dc0d
		bpl *+4
		ora #%10000010
		sta monitor_cia1_exp+$d

		stx $dd0e
		#lda0
		bit $dd0d
		bpl *+4
		ora #%10000001
		stx $dd0f
		bit $dd0d
		bpl *+4
		ora #%10000010
		sta monitor_cia2_exp+$d

		lda $d030		;C128?
		lsr
		bcs ccc64
		ldx #0
ndy		jsr getcrtcreg
		sta monitor_vdc_exp,x
		inx
		cpx #63
		bne ndy
		lda $d600
		sta monitor_vdc_exp+63
ccc64
                .if SUPERCPU=1
                lda monitor_oldbank_exp
                bpl nfreeze
                and #$7f
                sta monitor_oldbank_exp
                ldx #$0f
ll              lda $1d100,x
                sta monitor_vic_exp,x
                lda $1d110,x
                sta monitor_vic_exp+$10,x
                lda $1d160,x
                sta monitor_cia1_exp,x
                lda $1d180,x
                sta monitor_cia2_exp,x
                dex
                bpl ll
                lda $d250
                sta monitor_data_exp+1
nfreeze
		lda monitor_data_exp+1
                and #7
                sta monitor_01_exp
		.fi

		lda monitor_sr_exp
		and #$10
		beq nbrk
		lda monitor_pc_exp+1
		sec
		sbc #2
		sta monitor_pc_exp+1
		bcs nx
		#dec_l_a monitor_pc_exp
		gcc nx
nbrk		#inc_l_a monitor_data_exp+$318;fix blocked nmi
nx

		lda #1
		sta monitor_ripper_exp
mon_rip_start	lda #11
		sta $d011

		lda #<block_nmi
		sta $318
		lda #>block_nmi
		sta $319

		lda setup_moncolors_exp
		sta $d020
		ldx #0
		stx $d015
		stx $d01a
		lda #$fc
		sta $d030		;C128
		.if (SECOND_SID >> 8)=$d6
		lda $d030		;C128?
		lsr
		bcc *+5
		.fi
		stx SECOND_SID+$18	;Second SID
		stx $d01a
		stx $dc0e
		stx $dc0f
		stx $dd0e
		stx $dd0f
		dex
		stx $d019
		stx $d02f		;C128
		lda #20
		sta $d018
		lda #200
		sta $d016
		.if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                sta JiffyDos_internal_exp
                .fi
                
		lda $d030		;C128?
		lsr
		bcs ccc642
		ldx #0
ndy2		cpx #$1e
		beq e1
		cpx #$1f
		beq e1
		lda monitor_vdc_exp,x
		jsr setcrtcreg
e1		inx
		cpx #63
		bne ndy2
		lda $d600
		sta monitor_vdc_exp+63
ccc642

		ldx #14
ie		lda mon_peek,x
		sta $7e8,x
		dex
		bpl ie
                
                lda setup_moncolors_exp
                lsr
                lsr
                lsr
                lsr
		#sta_cfg_16
		txs
                pha
		jsr $fda3
		#lda0
		tay
nn		sta $2,y
		sta $200,y
		iny
		bne nn

		ldx #$3c
		ldy #$03
		stx $b2
		sty $b3
		ldy #$a0
		sty $c2
		ldx #$00
		jsr $fd8c		;Old memory checker

		jsr LWNO_set_new314vect
		dec $318		;disable nmi
		#stz_a ActualInput
                #stz_aa $291
                #stz_aa $cf
                lda #10
                sta $289
                sta $28c
		lda #3
		sta ActualOutput
                lda $d011
                bpl *-3
                pla
                sta $d021		;for kernal rev 2
		jsr $e536		;clear screen
                #sta_cfg_open
                lda setup_moncolors_exp
		sta $d021
		lda #27
		sta $d011

		ldx #0
		stx $dc0e
		stx m_inv
		stx m_prefetch
		.if SUPERCPU=1
		inx
		stx m_p2+2
		stx m_p4+2
		.fi
		bit $dc0d

		lda #128
		sta 650
                jsr LWOP_init_c128key
		lda monitor_dr_exp
		.byte SKIP2
eep		lda #8
		jsr mon_chdrv
		bcs eep
		lda #<$450
		sta m_p1
		lda #>$450
		sta m_p1+1

		lda monitor_sr_exp
		and #$10
		cmp #$10

		ldy #12
leo		lda txtmon,y
		bcc *+5
		lda txtbrk,y
		sta $400,y
		dey
		bpl leo

		lda monitor_01_exp
		pha

		jsr mon_printstat
		sty m_pos


		pla
		jsr u_b.in

		#sta_cfg_open
		lda monitor_tod_exp
		bne xxf
		ldx #6
xx		jsr mon_waitretrace
		dex
		beq xx

		ldx #1
		lda $dc08
		eor monitor_cia1_exp+8
		beq uy
		inx

uy		lda $dd08
		eor monitor_cia2_exp+8
		beq uy2
		inx
		inx
uy2		#stx_l_a monitor_tod_exp
xxf		#sta_cfg_16


loop		ldy m_pos
		lda 198
		bne hx
		lda (m_p1),y
		pha
		lda #15
		sta m_p3
d		ldy m_pos
		jsr mon_waitretrace
		inc m_p3
		lda m_p3
		and #15
		bne ig
		lda (m_p1),y
		eor #128
		sta (m_p1),y
ig		jsr $ea87
		lda 651
		lsr
		beq bd
		lda #2
		sta 651
bd		lda 198
		beq d
		pla
		ldy m_pos
		sta (m_p1),y
hx		dec 198

		#sta_cfg_open
		lda monitor_01_exp
		sta m_01
		#sta_cfg_16
		lda 631
		jsr xo
		#jmps loop


billc		#jword 29,b_right
		#jword 157,b_left
		#jword 147,b_clr
		#jword 19,b_home
		#jword 3,b_stop
		#jword 20,b_del
		#jword 141,b_shret
		#jword 148,b_ins
		#jword 133,b_f1
		#jword 136,b_f7
		#jword 18,b_rvson
		#jword 146,b_rvsoff
		#jword 13,b_ret
		#jword 17,b_down
		#jword 145,b_up
		#jword 134,b_f3
		#jword 135,b_f5
billce


xo		#switch billc,billce

		jsr tosccode
		eor m_inv
		sta (m_p1),y
;----------------------
;right
;
b_right		.proc
		cpy #39
		beq *+4
		inc m_pos
		rts
		.pend
;--------------------------
;f1
;
b_f1		.proc
		lda #134
		.byte SKIP2
;--------------------------
;f7
;
b_f7		.proc
		lda #135
		sta 631
		ldx #12
		stx 198
		rts
		.pend
;----------------------
;down
;
b_down		.proc
		sec
		.byte SKIP1
;--------------------------
;f5
;
b_f5		.proc
		clc
ll		lda m_p1
		ora m_p1+1
		eor #$c7
		beq le
		bcs s

ss2		jsr b_downin
		php
		jsr mon_scrolldown
		plp
		bcc ll
jmp_m_p4	.global
		jmp (m_p4)

le		jsr b_downin
		bcc br
		jsr scrldn
		jmp (m_p4)

br		jsr mon_cprevline

		lda m_p1+1
		cmp #4
		bcs le
scrldn		lda #<$7c0
		sta m_p1
		lda #>$7c0
		sta m_p1+1
s		jmp mon_scrolldown
		.pend

b_downin	.proc
		ldy #0
                .if SUPERCPU=1
		lda (m_p1),y
		tax
                .else
                lax (m_p1),y
                .fi
		iny
		jsr getaddr
		ldy #0
		lda m_p3
		cpx #":"
		beq ne1
		iny
		cpx #"."
		beq ne2
		iny
		cpx #","
		beq ne3
		cpx #">"
		beq ne3
		iny
		cpx #"'"
		beq ne4
		iny
		cpx #"["-64
		beq ne5
		iny
		cpx #"]"-64
		beq ne6
		clc
		rts

ne3		sta m_p2
		lda m_p3+1
		sta m_p2+1
		jsr mon_getbyte
		jsr mon_getoplen
		sec
		adc m_p3
		.byte SKIP2

ne1		adc #8-1
		.byte SKIP2

ne4
ne2		adc #32-1
		.byte SKIP2

ne5		adc #1-1
		#jmps fe

ne6		and #63
		cmp #60
		lda m_p3
		adc #3

fe		sta m_p2
		#lda0
		adc m_p3+1
		sta m_p2+1

mon_gprint	.proc
		lda mon_printlo,y
		sta m_p4
		lda mon_printhi,y
		sta m_p4+1
		sec
		rts
		.pend
;--------------------------
;left
;
b_left		.proc
		tya
		beq *+4
		dec m_pos
		rts
		.pend
;--------------------------
;up
;
b_up		.proc
		sec
		.byte SKIP1
;--------------------------
;f3
;
b_f3		.proc
		clc
ll		lda m_p1+1
		eor #4
		ora m_p1
		beq fel
		bcc ss2
		jmp mon_cprevline

ss2		jsr b_upin
		php
		jsr mon_cprevline
		plp
		bcc ll
		ldy #0
		jmp (m_p4)

fel		jsr b_upin
		bcc bty
		ldy #0
		lda #<$3c0
		sta m_p1
		lda #>$3c0
		sta m_p1+1
		jsr jmp_m_p4
bc		jmp mon_scrollup

bty		jsr mon_cnextline

		cmp #<$7e8
		lda m_p1+1
		sbc #>$7e8
		bcc fel
		ldy #40
		lda #$20
ld		sta $3bf,y
		dey
		bne ld
		geq bc
		.pend

b_upin		.proc
		ldy #0
                .if SUPERCPU=1
		lda (m_p1),y
		tax
                .else
                lax (m_p1),y
                .fi
		iny
		jsr getaddr
		ldy #0
		lda m_p3
		cpx #":"
		beq ne1
		iny
		cpx #"."
		beq ne2
		iny
		cpx #","
		beq ne3
		cpx #">"
		beq ne3
		iny
		cpx #"'"
		beq ne4
		iny
		cpx #"["-64
		beq ne5
		iny
		cpx #"]"-64
		beq ne6
		clc
		rts

ne3		lda #16
		sta m_tmp

do2		lda m_p3
		sec
		sbc m_tmp
		jsr mon_copy_p3p2sbc
		#stz_a m_tmp2
do		jsr mon_getbyte
		pha
		tax
		lda opc,x
		tax
		lda o_table+1,x
                #orasta_a m_tmp2
		pla
		jsr mon_getoplen
		sta m_p4
		sec
		adc m_p2
		sta m_p2
		bcc *+4
		inc m_p2+1

		cmp m_p3
		lda m_p2+1
		sbc m_p3+1
		bcc do
		ldx m_tmp
		cpx #8
		bcc *+6
		ldx m_tmp2
		bmi nig
		ldx m_p2
		cpx m_p3
		bne nig
		tax
		beq q
nig		dec m_tmp
		bne do2

q		lda m_p3
		clc
		sbc m_p4
		.byte SKIP2

ne1		sbc #8
		.byte SKIP2

ne4
ne2		sbc #32
		.byte SKIP2

ne5		sbc #1
		#jmps fe

ne6		and #63
		cmp #3
		lda m_p3
		sbc #3

fe		jsr mon_copy_p3p2sbc
		jmp mon_gprint
		.pend
;--------------------------
;clr
;
b_clr		.proc
		lda $d011
		bpl *-3

		lda #32
		ldx #24
		sta $400-24,x
		inx
		bne *-4
		sta $500-24,x
		inx
		bne *-4
		sta $600-24,x
		inx
		bne *-4
		sta $700-24,x
		inx
		bne *-4

;--------------------------
;home
;
b_home		.proc
		#stz_a m_p1
		#stz_aa m_pos
		lda #$04
		sta m_p1+1
;--------------------------
;stop
;
b_stop		.proc
		rts
		.pend
;--------------------------
;del
;
b_del		.proc
		tya
		beq loopx1
		dec m_pos
xh		lda (m_p1),y
		dey
		sta (m_p1),y
		iny
		iny
		cpy #40
		bne xh
		dey
		lda #32
		sta (m_p1),y
loopx1		rts
		.pend
;--------------------------
;shift return
;
b_shret		.proc
		#stz_a m_pos
		#stz_aa m_inv
		jmp mon_scrolldown
		.pend
;--------------------------
;insert
;
b_ins		.proc
		ldy #39
		lda (m_p1),y
		cmp #$20
		bne loopx2
igy		cpy m_pos
		beq ti
		dey
		lda (m_p1),y
		iny
		sta (m_p1),y
		dey
		bne igy
ti		lda #$20
		sta (m_p1),y
loopx2		rts
		.pend
;--------------------------
;rvsoff
;
b_rvsoff       	.proc
		#lda0
		.byte SKIP2
;--------------------------
;rvson
;
b_rvson		.proc
		lda #128
		sta m_inv
		rts
		.pend
;--------------------------
;return
;
b_ret		.proc
		sty m_tmp
		ldy #0
		sty m_pos
		sty m_inv

udu		lda (m_p1),y
		cmp #" "
		bne fo
		iny
		cpy #40
		bne udu
ex		jmp mon_scrolldown

fo		#switch utasc,utasce
		geq ex

utasc		#jword 13,u_m		;m
		#jword 9,u_i		;i
		#jword 4,u_d		;d
		#jword 10,u_j		;j
		#jword 24,exit		;x
		#jword 18,u_r		;r
		#jword 7,u_g		;g
		#jword 58,u_kpont	;:
		#jword 44,u_vesszo	;,
		#jword 62,u_vesszo	;>
		#jword 59,u_pvesszo	
		#jword 39,u_ijel	;'
		#jword 6,u_f		;f
		#jword 20,u_t		;t
		#jword 2,u_b		;b
		#jword 15,u_o		;o
		#jword 17,u_q		;q
		#jword 3,u_c		;c
		#jword 8,u_h		;h
		#jword 19,u_s		;s
		#jword 12,u_l		;l
		#jword 22,u_v		;v
		#jword 1,u_a		;a
		#jword 0,u_at		;@
		#jword 46,u_pont	;.
		#jword 5,u_e		;e
		#jword 27,u_balk	;[
		#jword 29,u_jobbk	;]
		#jword 31,u_bnyil	;<-
		#jword 30,u_fnyil	;^
		#jword 11,u_k		;k
		#jword 23,u_w		;w
                #jword 42,u_star	;*
utasce
		.pend
;---------------------------
;Check keyboard (and stop)
;
checkkey2	.proc
		#stz_a $dc00
checkkey1	.proc
		lda $dc01
		cmp $dc01
		bne *-6
		pha
		lda #$7f
		sta $dc00
		pla
		cmp #$7f
		rts
		.pend
;---------------------------
;X exit
;
exit		.proc
		jsr mon_copyback
		stx $dc0e
		stx $dc0f
		stx $dd0e
		stx $dd0f
		lda #127
		sta $dc0d
		sta $dd0d
		bit $dc0d
		bit $dd0d
		dex
		stx $d019
		inx
                .if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                stx JiffyDos_internal_exp
                .fi
dc		lda monitor_cia1_exp,x
		sta $dc00,x
		lda monitor_cia2_exp,x
		sta $dd00,x
		inx
		cpx #16
		bne dc
		lda monitor_tod_exp
		#sbc1
		lsr
		bcc lq
		ldx $dc08
		stx $dc08
lq		lsr
		bcc lq2
		ldx $dd08
		stx $dd08
lq2
		jsr mon_vicback
nf		lda monitor_data_exp+$100,x
		sta $100,x
		inx
		bne nf

                .if SUPERCPU=0
		lda monitor_pc_exp
		eor #>$fce2
		bne nr
		lda monitor_pc_exp+1
		eor #<$fce2
		bne nr
		lda monitor_sp_exp
		eor #$ff
		bne nr
		lda monitor_data_exp
		ora monitor_data_exp+1
		and #7
		bne nr
		#psr $fcec
		jmp gotobank0
nr
                .fi

		.if SUPERCPU=1
		lda monitor_sp_exp
		tcs
		.else
		ldx monitor_sp_exp
		txs
		.fi
		ldx #0
vyv		lda monitor_freeze_exp,x
		pha
		inx
		cpx #3
		bne vyv

                .if SUPERCPU=1
                sta scpu_hwenable
                lda monitor_d0b3_exp
                sta scpu_optimv2
                sta scpu_hwdisable
                .fi

		.if SUPERCPU=1
                lda monitor_data_exp+1
                .else
		lda monitor_data_exp
		eor #7
		ora monitor_data_exp+1
                .fi
		and #7
		.if SUPERCPU=1
		cmp #7
		.else
		cmp #5
		.fi
		bcs sd

		.if SUPERCPU=1
		pea $0101
		.else
		lda #$01
		pha
		pha
		.fi
		lda #%00100100
		pha

		pha
		pha
		pha

		lda #$40		;rti
		pha
		lda monitor_ac_exp	;lda #xx
		pha
		.if SUPERCPU=1
		pea $a901		;sta $01
		.else
		lda #$a9
		pha
		lda #$00                ;sta $00
		pha
		.fi
		tsx
		lda #$85
		pha

		pla
		pla
		pla
		pla
		pla

		pla
		pla
		pla

		txa
		tsx
		inx
		inx
		sta $100,x

		.if SUPERCPU=1
		lda monitor_data_exp+1
                .else
		lda monitor_data_exp
                .fi
		gcc ivi

sd		.if SUPERCPU=1
		lda monitor_data_exp+1
		sta $01
                .else
		lda monitor_data_exp
		sta $00
                .fi
		lda monitor_ac_exp
ivi		.if SUPERCPU=1
		xba
		.else
		sta help_irqa_exp
		.fi
		#ldatax monitor_xr_exp
		#ldy_l_a monitor_yr_exp
		#psr block_nmi
		lda #0		;no lda0!!! (xba)
		sta monitor_ripper_exp
		lda monitor_oldbank_exp
		cmp #3
		beq ey
		.if SUPERCPU=1
		xba
		.else
		lda help_irqa_exp
		.fi
		jmp exit2basic
ey		.if SUPERCPU=1
		xba
		.else
		lda help_irqa_exp
		.fi
		jmp switch_norm
		.pend

mon_vicback	.proc
		jsr mon_waitretrace
		lda monitor_vic_exp+$30
		and #%11111110
		sta $d030		;ide64 does not work with 2MHz
		ldx #$30
dc2		lda monitor_vic_exp-1,x
		sta $cfff,x
		dex
		bne dc2
		rts
		.pend

mon_copyback	.proc
		#sta_cfg_open
		lda #11
		sta $d011
		jsr mon_waitretrace
		.if SUPERCPU=1
                lda monitor_data_exp+1
                xba
                lda #$37
                sta monitor_data_exp+1
                .fi
		lda #2
		sta m_p1+1
		ldy #0
		sty m_p1
		sty m_p2
		lda #>(monitor_data_exp+$200)
		sta m_p2+1
		ldx #6
vy		#lda_zp_y m_p2
		sta (m_p1),y
		iny
		bne vy
		inc m_p1+1
		inc m_p2+1
		dex
		bne vy
                .if SUPERCPU=0
		txa
		geq ici
                .fi
ic		lda monitor_data_exp,x
ici		sta $00,x
		lda monitor_d800_exp,x
		sta $d900,x
		lsr
		lsr
		lsr
		lsr
		sta $d800,x
		lda monitor_d800_exp+$100,x
		sta $db00,x
		lsr
		lsr
		lsr
		lsr
		sta $da00,x
		inx
		bne ic
		.if SUPERCPU=1
                xba
                sta monitor_data_exp+1
                .fi
		rts
		.pend
;---------------------------
;W view
;
u_w		.proc
		jsr mon_copyback
		jsr mon_vicback
nf		lda monitor_data_exp+$100,x
		sta $100,x
		inx
		bne nf
		lda monitor_cia2_exp+2
		eor #3
		ora monitor_cia2_exp
		sta $dd00
		jsr checkn
		jsr checkn2
		jsr mon_waitretrace
		jmp monitor_start.mon_rip_start
		.pend
;---------------------------
;R Registers
;
u_r		.proc
		jsr mon_scrolldown
		jmp mon_printstat
		.pend
;---------------------------
;G Goto
;
u_g		.proc
		jsr get_addr2x
		jmp mon_exec
		.pend
;---------------------------
;Execute program in memory
;
mon_exec_mem	.proc
		#sta_cfg_open
		lda m_p3
		sta monitor_pc_exp+1
		lda m_p3+1
		sta monitor_pc_exp
		lda monitor_data_exp
		eor #7
		ora monitor_data_exp+1
		and #2
		beq hy2
		#ldatax monitor_sp_exp
		inx
		lda monitor_data_exp+$100,x
		cmp #<(r_mydoit+1)
		bne xe
		inx
		lda monitor_data_exp+$100,x
		cmp #>(r_mydoit+1)
		beq hy2
		dex
xe		dex
		lda #>(r_mydoit+1)
		sta monitor_data_exp+$100,x
		dex
		lda #<(r_mydoit+1)
		sta monitor_data_exp+$100,x
		dex
		#stx_l_a monitor_sp_exp
hy2		jmp exit
		.pend
;---------------------------
;Send M-? command to drive
;
drive_memcmd	.proc
		pha
		lda m_01
		jsr ieclisten		;listen
		lda #$6f
		jsr iecsecond		;second
		lda #"M"
		jsr iecout
		lda #"-"
		jsr iecout
		pla
		jmp iecout
		.pend
;---------------------------
;Execute program in drive
;
mon_exec_dr	.proc
		lda #"E"
		jsr drive_memcmd
		lda m_p3
		jsr iecout
		lda m_p3+1
		jsr iecout
		jsr iecunlisten		;unlisten
		jmp mon_scrolldown
		.pend
;---------------------------
;] Store sprite
;
u_jobbk		.proc
		lda #"]"+64
		.byte SKIP2
;---------------------------
;[ Store character
;
u_balk		.proc
		lda #"["-64
		jsr get_addr2store

		jsr get_binary2
		lda m_tmp2
		bpl io2
		bcs io
		jsr get_binary2
		bcs io
		jsr get_binary2
io		ldy #5
		gne sphr
io2		ldy #4
		gne sphr
		.pend
;---------------------------
;: Store hex
;
u_kpont		.proc
		lda #":"
		jsr get_addr2store
                lda #29
		cmp m_tmp
		bcs hex
                tay
                lda #$ff
                sta m_tmp2
                gne u_ijel.in

hex		lda #8
		sta m_tmp
xx1		jsr get_addr2
		bne xi1
		lda m_p3
		jsr mon_storebyte_inc
		dec m_tmp
		bne xx1

xi1		ldy #0
		geq sphr
;---------------------------
;, Store disassembly
;
u_vesszo	.proc
		tya
		bne vq
		lda (m_p1),y
		cmp #","
		bne vq
		lda m_tmp
                .if SUPERCPU=1
                cmp #18
                .else
		cmp #15
                .fi
		bcs u_a

vq		lda #","
		jsr get_addr2store

		jsr get_addr2
		bne xi2
		lda m_p3
		jsr mon_getoplen
		sta m_tmp
		#jmps vr
xx2		dec m_tmp
		bmi xi2
		jsr get_addr2
		bne xi2
vr		lda m_p3
		jsr mon_storebyte_inc
		#jmps xx2

xi2		ldy #2
;---------------------------
;Print here + space + position 6
;
sphr		.proc
		jsr mon_gprint
		ldy #0
		jsr jmp_m_p4
		jsr mon_scrolldown
		lda m_tmp2
		and #$7f
                cmp #"A"-64
                beq *+6
		cmp (m_p1),y
		bne b
		jsr mon_print_here
		lda #" "
		sta (m_p1),y
		gne topos6
b		jsr jmp_m_p4

topos6		.proc
		ldy #6
		sty m_pos
		rts
		.pend
;---------------------------
;. Store screencode
;
u_pont		.proc
		lda #"."
		.byte SKIP2
;---------------------------
;' Store ascii
;
u_ijel		.proc
		lda #"'"
		jsr get_addr2store

in	      	ldx #0
xf		iny
		lda m_tmp2
		lsr
		lda (m_p1),y
		bcc ik
		jsr fromsccode
ik		jsr mon_storebyte_inc
		cpy #37
		bne xf

                ldy #0
		lda m_tmp2
                bmi nn
		lsr
		iny
		bcc *+4
		ldy #3
nn		gne sphr
		.pend
;---------------------------
;; Change registers
;
u_pvesszo	.proc
		jsr get_addr2x
		#sta_cfg_open
		lda m_p3
		sta monitor_pc_exp+1
		lda m_p3+1
		sta monitor_pc_exp
		jsr get_addr2ddb
		sta monitor_ac_exp
		jsr get_addr2ddb
		sta monitor_xr_exp
		jsr get_addr2ddb
		sta monitor_yr_exp
		jsr get_addr2ddb
		sta monitor_sp_exp
		jsr get_addr2ddb
		jsr u_b.in
		#sta_cfg_open
		jsr get_addr2ddb
		jsr mon_chdrv
		ldx #8
igu		iny
		cpy #40
		beq ddb
		pha
		lda (m_p1),y
		lsr
		eor #24
		bne ddb2
		pla
		rol
		dex
		bne igu
		ora #%00100000
		sta monitor_sr_exp
		.byte SKIP2
xf		pla
ddb2		pla
ddb		#sta_cfg_16
		ldy #0
		geq mon_printstat.in

get_addr2ddb	jsr get_addr2
		bne xf
		lda m_p3
		rts
		.pend
;---------------------------
;F Fill
;
u_f		.proc
		jsr get_addr2x
		jsr cserep3p4
		jsr get_addr2x
		jsr cserep3p2
		jsr get_addr2x

uh		lda m_p3
		jsr mon_storebyte
		lda m_p4
		cmp m_p2
		lda m_p4+1
		sbc m_p2+1
		inc m_p4
		bne *+4
		inc m_p4+1
		bcc uh
		gcs mon_scrolldown
		.pend
;---------------------------
;T Transfer
;
u_t		.proc
		jsr get_addr2x
		jsr cserep3p2
		jsr get_addr2x
		jsr cserep3p4
		jsr get_addr2x
		jsr cserep3p4
		jsr in
		jmp mon_scrolldown
					;p2 start p3 end p4 new
in		lda m_p2
		cmp m_p4
		lda m_p2+1
		sbc m_p4+1
		bcs uh2

		lda m_p3
		pha
		sec
		sbc m_p2
		sta m_p3
		lda m_p3+1
		pha
		sbc m_p2+1
		sta m_p3+1

		php

		lda m_p3
		clc
		adc m_p4
		sta m_p4
		lda m_p3+1
		adc m_p4+1
		sta m_p4+1

		lda m_p2
		sta m_p3
		lda m_p2+1
		sta m_p3+1

		plp

		pla
		sta m_p2+1
		pla
		sta m_p2
		bcc enn

uh3		jsr mon_getbyte
		jsr mon_storebyte
		lda m_p3
		cmp m_p2
		lda m_p3+1
		sbc m_p2+1
		lda m_p2
		bne *+4
		dec m_p2+1
		dec m_p2
		lda m_p4
		bne *+4
		dec m_p4+1
		dec m_p4
		bcc uh3
enn		rts

uh2		jsr mon_getbyte
		jsr mon_storebyte
		lda m_p2
		cmp m_p3
		lda m_p2+1
		sbc m_p3+1
		inc m_p2
		bne *+4
		inc m_p2+1
		inc m_p4
		bne *+4
		inc m_p4+1
		bcc uh2
		rts
		.pend

cserep3p4	.proc
		ldx m_p3
		lda m_p4
		sta m_p3
		stx m_p4
		ldx m_p3+1
		lda m_p4+1
		sta m_p3+1
		stx m_p4+1
		rts
		.pend
;---------------------------
;BT Backtrace
;
u_bt		.proc
		jsr mon_scrolldown
                #psz m_p2
		#stz_a m_p3
		#sta_cfg_open
		lda monitor_sp_exp
		sta m_tmp
vwq		#sta_cfg_open
		inc m_tmp
		ldx m_tmp
		lda monitor_data_exp+$100,x
		sta m_tmp2
		sec
		sbc #2
		sta m_p2
		inx
		lda monitor_data_exp+$100,x
		sbc #0
		sta m_p2+1
		#sta_cfg_16
		jsr mon_getbyte
		cmp #$20		;jsr?
		bne fs
		inc m_tmp
		lda m_p3
		bne sc
		inc m_p3
		pla
		pla
                #psz m_p2
sc		lda m_p2+1
		jsr mon_printhex
		lda m_p2
		.byte SKIP2
fs		lda m_tmp2
		jsr mon_printhex
		lda #" "
		sta (m_p1),y
		iny
		cpy #36
		bcc vwq
		pla
		sta m_p2
		pla
		sta m_p2+1
		jmp mon_scrolldown
		.pend
;---------------------------
;B Select bank
;
u_b		.proc
		iny
		cpy #40
		beq xj

		lda (m_p1),y
		dey
		cmp #"T"-64
		beq u_bt

		jsr get_addr2x
		lda m_p3
		jsr in
		jmp mon_scrolldown

in		#sta_cfg_open
		sta monitor_01_exp
                sta m_01
		#sta_cfg_16
		ldx #mem-type+11
		cmp #8
		bcc norm
		ldx #drive-type+11
		cmp #31
		bcc norm
		ldx #vdc-type+11
		cmp #32
		bcc norm
		ldx #int-type+11
norm		#phy_a
		ldy #11
v		lda type,x
		dex
		sta mon_setprefetch,y
		dey
		bpl v
		#ply_a
xj		rts

type
drive		jmp mon_setprefetch_dr
		jmp mon_getbyte_dr
		jmp mon_storebyte_dr
		jmp mon_exec_dr
vdc		.byte $60,$60,$60
		jmp mon_getbyte_vdc
		jmp mon_storebyte_vdc
int		.byte $60,$60,$60
		jmp mon_getbyte_int
		.byte $60,$60,$60
mem		.byte $60,$60,$60
		jmp mon_getbyte_mem
		jmp mon_storebyte_mem
		jmp mon_exec_mem
		.pend
;---------------------------
;O Select device
;
u_o		.proc
		jsr get_addr2x
		lda m_p3
		#sta_cfg_open
		jsr mon_chdrv
		#sta_cfg_16
		bcs mon_chdrv.err
		jmp mon_scrolldown

mon_chdrv	.proc
		cmp #8
		bcc err
		cmp #31
		bcs err
		sta monitor_dr_exp
		sta ActualDev
		rts
err		sec
		rts
		.pend
;---------------------------
;Q Quit
;
u_q		.proc
		#sta_cfg_open
		lda #<$e37b
		sta monitor_pc_exp+1
		lda #>$e37b
		sta monitor_pc_exp
		lda #$2f
		sta monitor_data_exp
		lda #$37
		sta monitor_data_exp+1
		lda #<$ea31
		sta monitor_data_exp+$314
		lda #>$ea31
		sta monitor_data_exp+$315
		lda #$10
		sta $dc0e
		ldx #0
ls		lda $dc00,x
		sta monitor_cia1_exp,x
		inx
		cpx #13
		bne ls
dfs		lda cia-13,x
		sta monitor_cia1_exp,x
		inx
		cpx #16
		bne dfs
		geq exit
cia		.byte $81,$01,$08
		.pend
;---------------------------
;C Compare
;
u_c		.proc
		jsr get_addr2x
		jsr cserep3p2
		jsr get_addr2x
		jsr cserep3p4
		jsr get_addr2x
		jsr mon_scrolldown

uh4		jsr mon_getbyte
		sta m_tmp
		jsr cserep3p2
		jsr mon_getbyte
		eor m_tmp
		sta m_tmp
		jsr cserep3p2

		lda m_tmp
		beq ih
		cpy #40
		bne ih2
		jsr mon_scrolldown
ih2		ldx #m_p2
		jsr mon_printhexaddrsp
		jsr checkkey1
		beq ky
ih
		lda m_p2
		cmp m_p4
		lda m_p2+1
		sbc m_p4+1
		inc m_p2
		bne *+4
		inc m_p2+1
		inc m_p3
		bne *+4
		inc m_p3+1
		bcc uh4
		tya
		beq ky
		jmp mon_scrolldown
ky		rts
		.pend

cserep3p2	.proc
		ldx m_p2
		lda m_p3
		sta m_p2
		stx m_p3
		ldx m_p2+1
		lda m_p3+1
		sta m_p2+1
		stx m_p3+1
		rts
		.pend
;---------------------------
;H Hunt
;
u_h		.proc
		jsr get_addr2x
		jsr cserep3p2
		jsr get_addr2x
		jsr cserep3p4

		ldx #0

xcc		jsr get_addr2
		cpy #40
		bcs hii
		cmp #32
		beq opo
		cmp #34
		beq tcg
		cmp #"?"
		bne hii
		geq kx

tcg		iny
		cpy #40
		bcs hii
		lda (m_p1),y
		cmp #34
		beq xcc
		jsr fromsccode
		sta monitor_buffer,x
		#lda0
		sta monitor_buffer+40,x
		inx
		gne tcg

opo		lda m_p3
		sta monitor_buffer,x
		#lda0
kx		sta monitor_buffer+40,x
		inx
		gcs xcc

hii		txa
		sta m_tmp
		bne uh5
xg		rts

uh5		lda m_p2+1
		sta m_p3+1
		lda m_p2
		sta m_p3
uh5k		ldx #0
uh6		stx m_tmp2
		jsr mon_getbyte
		ldx m_tmp2
		cmp monitor_buffer,x
		beq ih4x
		lda monitor_buffer+40,x
		beq ih4
ih4x		inc m_p2
		bne *+4
		inc m_p2+1
		inx
		cpx m_tmp
		bne uh6
		cpy #40
		bne ih3
		jsr mon_scrolldown
ih3		ldx #m_p3
		jsr mon_printhexaddrsp
		jsr checkkey1
		beq xg

ih4		ldx m_p3
		cpx m_p4
		lda m_p3+1
		sbc m_p4+1
		bcs kq
		cpx #255
		inx
		stx m_p2
		stx m_p3
		#lda0
		adc m_p3+1
		sta m_p2+1
		sta m_p3+1
		#jmps uh5k

kq		jmp mon_scrolldown
		.pend
;---------------------------
;S Save
;
u_s		.proc
		jsr mon_getfname
		bcc *+3
		rts
		jsr get_addr2
		bne bi			;freeze!
		jsr cserep3p2
		jsr get_addr2x

		#lda0
		.byte SKIP2
bi		lda #1
		sta m_tmp

		lda #1
		jsr mon_open
		jsr sii
		jsr newchkout_t
		jsr sii
		lda m_tmp
		beq xxf

		jsr mon_frzcomm
dy		#sta_cfg_open
		dey
		#lda_zp_y m_p2
		#sta_cfg_16
		jsr dchrout
		tya
		bne dy
		dec m_p2+1
		dex
		bne dy
		lda #4
		sta m_01
		stx m_p2
		stx m_p2+1
		dex
		stx m_p3
                ldx #$07
		stx m_p3+1
                jsr writeb
		lda #>$0800
		ldy #>$c800
		sta m_p2+1
		ldx #0
		lda #m_p2
		jsr newwrite_t_mon
                ldx #$d0
                bcc nek
                ldx ActualLogical
                jsr newchkout_t
                ldx #$08
nek		stx m_p2+1
                ldx #$00
		stx m_p2
		dex
		stx m_p3
		stx m_p3+1
		gmi uhu

xxf		lda m_p2
		jsr dchrout
		lda m_p2+1
		jsr dchrout

uhu		jsr writeb
		jsr newclrchn_t
		jsr sii
		lda ActualLogical
		jsr newclose_t
		jsr sii
		jsr mon_scrolldown
six		lda #"O"-64
		sta (m_p1),y
		iny
		lda #"K"-64
		sta (m_p1),y
		jmp mon_scrolldown

writeb		jsr mon_getbyte
		jsr dchrout
		lda m_p2
		cmp m_p3
		lda m_p2+1
		sbc m_p3+1
		inc m_p2
		bne dq
		php
		jsr checkkey1
		beq u_s.sii4
		plp
		inc m_p2+1
dq		bcc writeb
		rts

dchrout		jsr newchrout_t
sii		bcs sii2
		inc $d020
		dec $d020
		pha
		lda Status
		cmp #1
		pla
		bcs sii2
		rts
sii2		pla
sii4		pla
sii3		jsr newclrchn_t
		lda ActualLogical
		jmp newclose_t
		.pend
;---------------------------
;L Load
;
u_l		.proc
		#lda0
		.byte SKIP2
;---------------------------
;V Verify
;
u_v		.proc
		lda #1
		sta VerifyFlag
		jsr mon_getfname
		bcc *+3
		rts
		#stz_a m_tmp
		jsr get_addr2
		bne nh
		inc m_tmp
		jsr cserep3p4
nh
		#lda0
		sta m_tmp2
in		jsr mon_open
		jsr u_s.sii
		jsr newchkin_t
		jsr u_s.sii
		lda m_tmp2
		beq ini

		jsr mon_frzcomm
dy		jsr newchrin_t
		#sta_cfg_open
		dey
		#sta_zp_y m_p2
		#sta_cfg_16
		jsr u_s.sii
		tya
		bne dy
		dec m_p2+1
		dex
		bne dy
		stx m_p2
		stx m_p4
		stx m_p4+1
		ldy #>$800
		tya
		jsr read
		bcs br1
		lda #>$0fff
		jsr copy
		lda #>$0800
		ldy #>$c800
		jsr read
		lda #>$e000
		ldy #>$1000
		jsr read
		ldx #>$d000
		lda #>$efff
		jsr copy
		lda #>$e000
		ldy #>$2000
		jsr read
		stx m_p3
		stx m_p3+1
		dex
		stx m_p4
		stx m_p4+1
		php
		jmp dgu

copy		sta m_p3+1
		stx m_p4+1
		lda #$ff
		sta m_p3
		gne u_t.in

read		sta m_p2+1
		ldx #0
		lda #m_p2
		jmp newread_t

ini		jsr newchrin_t
		sta m_p2
		jsr u_s.sii
		jsr newchrin_t
		sta m_p2+1
		jsr u_s.sii
                #jmps br2

br1             #stz_a m_p2+1
                ldx ActualLogical
                jsr newchkin_t

br2     	ldx m_p4
	    	ldy m_p4+1
		lda m_tmp
		bne uhux
		ldx m_p2
		stx m_p4
		ldy m_p2+1
		sty m_p4+1
uhux		sty m_p3+1
		stx m_p3
		ldy #0

uhu2		lda VerifyFlag
		beq tc
		lda m_p4
		sta m_p2
		lda m_p4+1
		sta m_p2+1
		jsr mon_getbyte
		sta m_tmp
		jsr newchrin_t
		php
		cmp m_tmp
		beq dgu
		tya
		beq *+6
		cpy #40
		bne *+5
		jsr mon_scrolldown
		ldx #m_p2
		jsr mon_printhexaddrsp
		#jmps dgu

tc		jsr newchrin_t
		php
		jsr mon_storebyte
dgu		plp
		lda Status
		eor #$40
		beq sixq
		jsr u_s.sii
		inc m_p4
		bne uhu2
		jsr checkkey1
		beq u_s.sii3
		inc m_p4+1
		#jmps uhu2

sixq		sta Status
		jsr newclrchn_t
		jsr u_s.sii
		lda ActualLogical
		jsr newclose_t
		jsr u_s.sii
		jsr mon_scrolldown
		ldx #m_p3
		jsr mon_printhexaddr
		lda #"-"
		ldx #m_p4
		jsr mon_print_hereaddr
		lda #" "
		sta (m_p1),y
		iny
		jmp u_s.six		;print "OK"
		.pend
;---------------------------
;K Defreeze
;
u_k		.proc
		jsr mon_getfname
		bcc *+3
		rts

		lda #4
		sta m_01
		sta m_tmp2
		#lda0
		sta VerifyFlag
		sta m_tmp
		geq u_v.in
		.pend
;---------------------------
;A Assembly
;
u_a		.proc

		.if SUPERCPU=1
offs		= 58
		.else
offs		= 55
		.fi

		jsr get_addr2x
		jsr cserep3p4
nj		iny
		cpy #37
		bcs vvv
		ldx #3
sc		lda (m_p1),y
		beq nj
		cmp #"Z"-63
		bcs nj
		iny
		dex
		bne sc
		lda (m_p1),y
		cmp #" "
		bne nj
		dey
		dey
		dey
		stx m_tmp
dc		lda (m_p1),y
		sta 4+monitor_buffer,x
		inx
		iny
		cmp #"$"
		bne ibb
		lda m_tmp
		bne ibb
		inc m_tmp
		#phy_a
		jsr getaddr
		#ply_a
		jsr isnumnext
		bcs ibb
		jsr isnumnext
		bcc ibb2
		jsr sml
		lda #"0"
		sta 4+monitor_buffer-2,x
		inx
		gcs ibb

sml		lda 4+monitor_buffer-1,x
		sta 4+monitor_buffer,x
		lda 4+monitor_buffer-2,x
		sta 4+monitor_buffer-1,x
vvv		rts

sml2		lda 4+monitor_buffer-3,x
		sta 4+monitor_buffer-2,x
		lda 4+monitor_buffer-4,x
		sta 4+monitor_buffer-3,x
                rts

ibb2		jsr isnumnext
                .if SUPERCPU=1
                bcs ibb4
                .else
		bcs ibb
                .fi
	     	jsr isnumnext
                .if SUPERCPU=1
                bcc ibb3
                .else
		bcc ibb
		.fi
		jsr sml
		jsr sml2
		lda #"0"
		sta 4+monitor_buffer-4,x
		inx
                .if SUPERCPU=1
                gcs ibb

ibb3		jsr isnumnext
		bcs ibb
		jsr isnumnext
		bcc ibb
		jsr sml
		jsr sml2
		lda 4+monitor_buffer-5,x
		sta 4+monitor_buffer-4,x
		lda 4+monitor_buffer-6,x
		sta 4+monitor_buffer-5,x
		lda #"0"
		sta 4+monitor_buffer-6,x
		inx
                gcs ibb

ibb4		cmp #","
		bne ibb
                jsr isnumnext
                cmp #"$"
		bne ibb
		#phy_a
		jsr getaddrnc
		#ply_a
                jsr isnumnext
		.fi

ibb		cpy #40
		bcc dc

		#psz m_p1
		lda #$4c
		sta mon_getbyte
		lda #<mon_fake_getbyte
		sta mon_getbyte+1
		lda #>mon_fake_getbyte
		sta mon_getbyte+2
		lda #$60
		sta mon_setprefetch

		lda #<(monitor_buffer+40)
		sta m_p1
		lda #>(monitor_buffer+40)
		sta m_p1+1

		#stz_a $100
		lda m_p3
		sta $101
		lda m_p3+1
		sta $102
                .if SUPERCPU=1
                lda m_p3+2
		sta $103
                .fi
                
meg		#stz_a m_p2
		ldx $100
                ldy opc,x
                lda o_table,y
                and #31
                cmp 4+monitor_buffer
                bne bt

                .if SUPERCPU=1
                cpx #$80
                beq bbra
                cpx #$82
                beq bbra
                cpx #$62
                beq bbra
                .fi
                txa
		and #%00011111
		cmp #$10		;branch?
		bne nth
bbra		stx $104
		lda #4
		sta m_p2
nth
		lda #$01
		sta m_p2+1
		ldy #5
		jsr mon_printd2

		ldx #0
		lda m_p2
		cmp #6
		bcc xfv

		lda #128
                .if SUPERCPU=1
                ldy #23
                .else
		ldy #20
                .fi
		.byte SKIP1
xfv		txa
		sta (m_p1),y	          ;012345678901234567890
dd		lda monitor_buffer+offs,x ;,1234 11 22 33 LDA $1234
		clc
		bmi ok
		lda 4+monitor_buffer,x
		eor #" "
		bne hb
		lda monitor_buffer+offs,x
		beq ok
hb		lda monitor_buffer+offs,x
		cmp 4+monitor_buffer,x
		bne bt
		inx
		gpl dd
bt		inc $100
		bne meg
err		sec
ok
		php
		lda $100
                ldx m_p3
                .if SUPERCPU=1
                cmp #$80
                beq bbra2
                cmp #$82
                beq bbra3
                cmp #$62
                beq bbra3
                .else
		cmp #$1a
		bne nnop
		lda #"E"-64
		sta monitor_buffer+offs-9
		lda #$ea
		sta $100
nnop		
		.fi
		and #%00011111
		cmp #$10
		bne hi
bbra2
		lda #2-1
		adc m_p4
		sta m_tmp2
		#lda0
		adc m_p4+1
		sta m_tmp2+1

		lda m_p3
		sec
		sbc m_tmp2
		tax
		lda m_p3+1
		sbc m_tmp2+1
		bcc ms
		bne errc
		txa
		bmi errc
		gpl mege
ms		cmp #$ff
		bne errc
		txa
		bmi mege
errc		plp
		sec
		php
                .if SUPERCPU=1
                #jmps mege
bbra3
		lda #3-1
		adc m_p4
		sta m_tmp2
		#lda0
		adc m_p4+1
		sta m_tmp2+1

		lda m_p3
		sec
		sbc m_tmp2
		tax
		lda m_p3+1
		sbc m_tmp2+1
                pha
                ldy #9
		jsr mon_printhex
                ldy #23
                lda m_p3+1
		jsr mon_printhex
                pla
                sta m_p3+1
		#jmps tu
                .fi
                      
mege		ldy #9
		jsr mon_printhex
                .if SUPERCPU=1
                ldy #23
                .else
		ldy #20
                .fi
		lda m_p3+1
		jsr mon_printhex
tu      	lda m_p3
		jsr mon_printhex
		stx m_p3
hi		plp

		pla
		sta m_p1
		pla
		sta m_p1+1
		bcs vvv2

                lda m_p4
                sta m_p2
                lda m_p4+1
                sta m_p2+1
		lda $100
		ldy #32
		jsr mon_storebyte_inc
                .if SUPERCPU=1
		cpy monitor_buffer+offs-9
		beq vv
		lda m_p3
		jsr mon_storebyte_inc
                .fi
		cpy monitor_buffer+offs-6
		beq vv
                .if SUPERCPU=1
                lda m_p3+1
                .else
		lda m_p3
                .fi
		jsr mon_storebyte_inc
		cpy monitor_buffer+offs-3
		beq vv
                .if SUPERCPU=1
                lda m_p3+2
                .else
		lda m_p3+1
                .fi
		jsr mon_storebyte_inc
vv
		jsr vvv2
                ldy #0
                lda (m_p1),y
                ldy #2
                sta m_tmp2
		cmp #","
                beq *+4
                ldy #6
		jmp sphr

vvv2		#sta_cfg_open
		lda monitor_01_exp
		jmp u_b.in

isnumnext	lda (m_p1),y
		beq ib
		cmp #"G"-64
		bcc ob
		cmp #"0"
		bcc ib
		cmp #"9"+1
		.byte SKIP1
ib		sec
ob
		sta 4+monitor_buffer,x
		inx
		iny
		rts

mon_fake_getbyte
		ldx m_p2
		lda $100,x
		rts
		.pend
;---------------------------
;@ Disk command
;
u_at		.proc
		sty m_tmp
		jsr mon_getfname
		bcc lu
		#stz_a ActualFNLength
		lda #" "
		ldy #40
tr		dey
		cpy m_tmp
		beq lu
		cmp (m_p1),y
		beq tr
		sty m_tmp2
		ldx #0
		ldy m_tmp
ic		iny
		lda (m_p1),y
		jsr fromsccode
		sta monitor_buffer,x
		inx
		cpy m_tmp2
		bcc ic
		jsr mon_getfnameset

lu		lda ActualFNLength
		beq tii
		lda monitor_buffer
		eor #"$"
		bne tii
		.byte SKIP2
tii		lda #15

		jsr mon_open
		jsr u_s.sii
		jsr newchkin_t
		jsr u_s.sii
		lda ActualSecAddr
		and #15
		beq dir
		jsr mon_scrolldown
ti		jsr newchrin_t
		ldx Status
		bne ti2
		jsr tosccode
		sta (m_p1),y
		iny
		cpy #40
		bne ti
ti2		jsr mon_scrolldown
ti2x		jmp u_s.sii3

dchrin		jsr newchrin_t
		jmp u_s.sii

dir		jsr dchrin
		jsr dchrin

		lda #128
dirlp		sta m_tmp
		jsr mon_scrolldown
		lda m_p1
		sta m_p2
		lda m_p1+1
		sta m_p2+1
		jsr dchrin
		jsr dchrin
		jsr dchrin
		tax
		jsr dchrin
		jsr numtoasc16

		lda m_tmp
		bpl tw
		jsr dchrin

tw		lda #32
tii2		sta (m_p1),y
xd2		iny
xd		jsr checkkey1
		beq ti2x
		ldx Status
		bne ti2
		jsr dchrin
		beq dirlp
		cmp #20
		bne bu
		dey
		bpl xd
		gmi xd2
bu		jsr tosccode
		eor m_tmp
		cpy #40
		bne tii2
		geq dirlp
		.pend
;---------------------------
;<- Remember position
;
u_bnyil		.proc
		jsr get_addr2
		beq xu
		clc
xu		ldx m_tsp
		lda m_p3
		bcs *+4
		lda m_p2
		sta m_tstack,x
		lda m_p3+1
		bcs *+4
		lda m_p2+1
		sta m_tstack+8,x
		inx
		txa
		and #7
		sta m_tsp
		jsr get_addr2
		beq xu
		jmp mon_scrolldown
		.pend
;---------------------------
;^ Restore position
;
u_fnyil		.proc
		jsr poptsp
		sta m_p2
		lda m_tstack+8,x
		sta m_p2+1
		jmp mon_scrolldown
		.pend

;---------------------------
;* RAM/ROM
;
u_star	       	jsr mon_scrolldown
		#sta_cfg_open
		lda monitor_01_exp
		#sta_cfg_16
                cmp #0
                beq vs
                cmp #4
vs              php
                bne lf
                lda #7
                .byte SKIP2
lf              lda #4
		jsr u_b.in

		lda #"R"-64
		sta (m_p1),y
		iny
		lda #"A"-64
                plp
                bne *+4
                lda #"O"-64
		sta (m_p1),y
		iny
		lda #"M"-64
		sta (m_p1),y
		jmp mon_scrolldown



;---------------------------
;Pop tsp
;
poptsp		.proc
		ldx m_tsp
		dex
		txa
		and #7
		sta m_tsp
		tax
		lda m_tstack,x
		rts
		.pend
;---------------------------
;E Edit
;
u_e		.proc
		iny
		cpy #40
		beq xf

		lda (m_p1),y
		cmp #"C"-64
		beq u_ec
		cmp #"S"-64
		beq u_es
xf		rts
;---------------------------
;I Ascii
;
u_i		.proc
		cpy #39
		bcs up
		iny
		lda (m_p1),y
		cmp #"*"
		beq u_j
		dey
up		ldx #3
		.byte SKIP2
;---------------------------
;D Disassembly
;
u_d		.proc
		ldx #2
		.byte SKIP2
;---------------------------
;ES Sprite
;
u_es		.proc
		ldx #5
		.byte SKIP2
;---------------------------
;EC binary
;
u_ec		.proc
		ldx #4
		.byte SKIP2
;---------------------------
;J Screencode
;
u_j		.proc
		ldx #1
		.byte SKIP2
;---------------------------
;M Hex
;
u_m		.proc
		ldx #0
		jsr checkn
		lda mon_printlo,x
		sta m_p4
		lda mon_printhi,x
		sta m_p4+1
		lda len,x
		tax
		clc
		adc m_p2
		sta m_p3
		#lda0
		adc m_p2+1
		sta m_p3+1
		lda #32
xp		iny
		cmp (m_p1),y
		beq xp2
		stx m_p2
		jsr getaddr
		pha
		#ldatax m_p3
		clc
		adc m_p2
		sta m_p3
		stx m_p2
		lda m_p3+1
		sta m_p2+1
		adc #0
		sta m_p3+1
		pla
		eor #32
		beq xi
		cmp #"-"^32
		bne ll
xi		tax
		jsr get_addr2
		beq ll
		txa
		beq ll
		lda #$ff
		sta m_p3
		sta m_p3+1
		gne ll

xp2		cpy #39
		bne xp
ll		jsr mon_scrolldown
		#psz m_p2
		jsr jmp_m_p4
		pla
		cmp m_p2
		pla
		sbc m_p2+1
		bcs sss
		jsr checkkey1
		beq sss
		cmp #$fb
		beq bu
		lda m_p1
		cmp #$c0
		bcc bu
		lda m_p1+1
		cmp #$07
		bcc bu
		jsr checkkey2
		cmp #$ff
		beq bu
		jsr checkn
		jsr checkn2
		jsr checkkey1
		beq sss
		jsr checkn
bu		lda m_p2
		cmp m_p3
		lda m_p2+1
		sbc m_p3+1
		bcc ll
sss		jmp topos6

len		.byte 32,128,8,128,8,63
		.pend

checkn		jsr checkkey2
		cmp #$ff
		bne checkn
		rts
checkn2		jsr checkkey2
		cmp #$ff
		beq checkn2
		rts

mon_printlo	.byte <mon_printm,<mon_printj,<mon_printd,<mon_printi,<mon_printec,<mon_printes,<mon_printa
mon_printhi	.byte >mon_printm,>mon_printj,>mon_printd,>mon_printi,>mon_printec,>mon_printes,>mon_printa
;---------------------------
;Print hex number with space
;
mon_printhexaddrsp .proc
		lda #32
;---------------------------
;Print address
;
mon_print_hereaddr .proc
		.byte SKIP2
;---------------------------
;Print mode and address (:1000)
;
mon_print_here	.proc
		ldx #m_p2
		sta (m_p1),y
		iny
;---------------------------
;Print hex number
;
mon_printhexaddr .proc
		lda $01,x
		jsr mon_printhex
		lda $00,x
		#jmps mon_printhex

mon_printhexsp	.proc
		pha
		lda #32
		sta (m_p1),y
		iny
		pla
mon_printhex	.proc
		pha
		lsr
		lsr
		lsr
		lsr
		jsr h
		pla
		and #15
h		cmp #10
		bcc *+4
		sbc #$39
		adc #$30
		sta (m_p1),y
		iny
		rts
		.pend
;---------------------------
;Print processor status
;
mon_printstat	.proc
		ldy #0
l		lda txtstat,y
		sta (m_p1),y
		iny
		cpy #32
		bne l
		jsr mon_clearend
		jsr mon_scrolldown
in		#sta_cfg_open
		lda monitor_pc_exp+1
		sta m_p2
		lda monitor_pc_exp
		sta m_p2+1

		lda #";"
		jsr mon_print_here
                lda #4
                sta m_tmp
vsc		ldx m_tmp
		lda sor,x
                tax
                lda monitor_01_exp,x
		jsr mon_printhexsp
                dec m_tmp
                bpl vsc
		lda ActualDev
		jsr mon_printhexsp
		lda #" "
		sta (m_p1),y
		iny
		lda monitor_sr_exp
		ldx #8
fg		asl
		pha
		lda #"0" >> 1
                rol
		sta (m_p1),y
		iny
		pla
		dex
		bne fg
		#sta_cfg_16
		jsr mon_clearend
		jmp mon_scrolldown

sor		.byte monitor_01_exp-monitor_01_exp
		.byte monitor_sp_exp-monitor_01_exp
		.byte monitor_yr_exp-monitor_01_exp
		.byte monitor_xr_exp-monitor_01_exp
		.byte monitor_ac_exp-monitor_01_exp
		
txtstat		.byte " ","A"-64,"D"-64,"D"-64,"R"-64," ","A"-64,"C"-64," ","X"-64,"R"-64," ","Y"-64,"R"-64," ","S"-64,"P"-64," ","B"-64,"K"-64," ","D"-64,"R"-64," "
		.byte "N"-64,"V"-64,"-","B"-64,"D"-64,"I"-64,"Z"-64,"C"-64
		.pend
;---------------------------
;Scroll downward
;
mon_scrolldown	.proc
		lda m_p1
		ora m_p1+1
		eor #$c7
		bne eu

		ldx #15
iy		jsr checkkey1
		cmp #$fb
		bne tp
		jsr mon_waitretrace
		dex
		bne iy

tp		lda $d011
		bpl *-3

		ldy #40
ni		lda $400,y
		sta $400-40,y
		iny
		lda $400,y
		sta $400-40,y
		iny
		bne ni
ni2		lda $500,y
		sta $500-40,y
		iny
		lda $500,y
		sta $500-40,y
		iny
		bne ni2
ni3		lda $600,y
		sta $600-40,y
		iny
		lda $600,y
		sta $600-40,y
		iny
		bne ni3
ni4		lda $700,y
		sta $700-40,y
		iny
		lda $700,y
		sta $700-40,y
		iny
		cpy #232
		bne ni4

		ldy #40
		lda #32
nn		sta $7bf,y
		dey
		bne nn
		rts

eu		ldy #0

mon_cnextline	.proc
		lda m_p1
		clc
		adc #40
		sta m_p1
		bcc *+4
		inc m_p1+1
		rts
		.pend
;---------------------------
;Scroll upward
;
mon_scrollup	.proc
		lda #<$400
		sta m_p1
		lda #>$400
		sta m_p1+1

		lda $d011
		bpl *-3

		ldy #39
ni1
		lda $400+480,y
		sta $7c0,y

		lda $400+440,y
		sta $400+480,y

		lda $400+400,y
		sta $400+440,y

		lda $400+360,y
		sta $400+400,y

		lda $400+320,y
		sta $400+360,y

		lda $400+280,y
		sta $400+320,y

		lda $400+240,y
		sta $400+280,y

		lda $400+200,y
		sta $400+240,y

		lda $400+160,y
		sta $400+200,y

		lda $400+120,y
		sta $400+160,y

		lda $400+80,y
		sta $400+120,y

		lda $400+40,y
		sta $400+80,y

		lda $400,y
		sta $400+40,y

		lda $3c0,y
		sta $400,y
		dey
		bpl ni1

		ldy #39
ni2
		ldx $7c0,y

		lda $400+920,y
		sta $7c0,y

		lda $400+880,y
		sta $400+920,y

		lda $400+840,y
		sta $400+880,y

		lda $400+800,y
		sta $400+840,y

		lda $400+760,y
		sta $400+800,y

		lda $400+720,y
		sta $400+760,y

		lda $400+680,y
		sta $400+720,y

		lda $400+640,y
		sta $400+680,y

		lda $400+600,y
		sta $400+640,y

		lda $400+560,y
		sta $400+600,y

		lda $400+520,y
		sta $400+560,y

		txa
		sta $400+520,y

		dey
		bpl ni2
		iny
		rts
		.pend
;---------------------------
;Print memory as hex
;
mon_printm	.proc
		lda m_p3
		pha
		lda #":"
		jsr mon_print_here
		lda #8
		jsr mon_setprefetch

		#stz_a m_p3
dp2		jsr mon_getbyte
		inc m_p2
		bne *+4
		inc m_p2+1
		ldx m_p3
		sta monitor_buffer,x
		inc m_p3
		jsr mon_printhexsp
		cpy #29
		bne dp2
		lda #" "
		sta (m_p1),y
		iny
		ldx #0
dp		lda monitor_buffer,x
		jsr tosccode
		sta (m_p1),y
		iny
		inx
		cpx #8
		bne dp
		pla
		sta m_p3
		jmp mon_clearend
		.pend
;---------------------------
;Print memory as screencode
;
mon_printj	.proc
		lda #"."
		.byte SKIP2
;---------------------------
;Print memory as ascii
;
mon_printi	.proc
		lda #"'"
		cmp #"."
		ror m_tmp
		jsr mon_print_here
		lda #32
		jsr mon_setprefetch

		lda #" "
		sta (m_p1),y
ny		iny
		jsr mon_getbyte
		inc m_p2
		bne *+4
		inc m_p2+1
		bit m_tmp
		bmi ua
		jsr tosccode
ua		sta (m_p1),y
		cpy #37
		bne ny
		iny
		gne mon_clearend
		.pend
;---------------------------
;Print memory as character
;
mon_printec	.proc
		lda #"["-64
		jsr in
		jmp mon_clearend

in		jsr mon_print_here

		lda #" "
		sta (m_p1),y
		iny

in2		jsr mon_getbyte
		inc m_p2
		bne *+4
		inc m_p2+1

		ldx #8
fg		asl
		pha
		lda #"."
		bcc *+4
		lda #"#"
		sta (m_p1),y
		iny
		pla
		dex
		bne fg
		rts
		.pend
;---------------------------
;Print memory as sprite
;
mon_printes	.proc
		lda #"]"-64
		jsr mon_printec.in
		jsr mon_printec.in2
		jsr mon_printec.in2
		lda m_p2
		and #63
		cmp #63
		bne mm
		inc m_p2
		bne *+4
		inc m_p2+1
mm		jmp mon_clearend
		.pend
;---------------------------
;Calculate opcode length
;
mon_getoplen	.proc
		lsr
		tax
		lda a_mod,x
		bcc o
		lsr
		lsr
		lsr
		lsr
                sec
o		and #15
		rol
		tax
		lda a_len,x
                and #3
		rts
		.pend
;---------------------------
;Print memory as disassembly
;
mon_printa	.proc
		lda #"A"-64
                .byte SKIP2
mon_printd	.proc
		lda #","
		jsr mon_print_here
		jsr mon_printd2
		jmp mon_clearend

mon_printd2	.proc
		#psz m_p3

		jsr mon_getbyte
		tax
		jsr phex
		lda opc,x
		sta m_p3+1
		txa
		jsr mon_getoplen
		sta m_p3
                lda a_len,x
		pha
		dec m_p3
		bmi ip
		jsr mon_getbyte
		sta monitor_buffer+1
		jsr phex
		dec m_p3
		bmi ip
		jsr mon_getbyte
		sta monitor_buffer+2
		jsr phex
                .if SUPERCPU=1
		dec m_p3
		bmi ip
		jsr mon_getbyte
		sta monitor_buffer+3
		jsr phex
                .fi
ip
		lda #32
ii		sta (m_p1),y
		iny
                .if SUPERCPU=1
                cpy #18
                .else
		cpy #15
                .fi
		bne ii

		ldx m_p3+1
		lda o_table+1,x
		sta m_p3+1
		lda o_table,x
		jsr mon_printopc
		lda #32
		sta (m_p1),y
		iny

		pla
                sta m_p3
                lsr
                lsr
                and #3
                beq tu
                tax
                lda a_first-1,x
                sta (m_p1),y
                iny
tu              lda m_p3
		cmp #$f0		;relative
		and #3
                beq n4i
                sta m_p3+1

		bcc n5
                sta m_p3
                
                .if SUPERCPU=1
                cmp #1
                beq nor
                lda monitor_buffer+1
                clc
                adc m_p2
                sta monitor_buffer+1
                lda monitor_buffer+2
                adc m_p2+1
                #jmps dc
nor                
		.fi
                
                inc m_p3+1

		#ldatax monitor_buffer+1
		clc
		adc m_p2
		sta monitor_buffer+1
		txa
		bmi s
		#lda0
		adc m_p2+1
		#jmps dc

s		lda m_p2+1
		sbc #0
dc		sta monitor_buffer+2

n5		lda #"$"
		sta (m_p1),y
		iny

tt      	ldx m_p3+1
		lda monitor_buffer,x
		jsr mon_printhex
                dec m_p3+1
                bne tt

		lda m_p3
                lsr
                lsr
                lsr
                lsr
                beq n4
                .if SUPERCPU=1
                cmp #2
                bne nkn
                dey
                dey
                lda #","
		sta (m_p1),y
		iny
                lda #"$"
		sta (m_p1),y
		iny
                lda monitor_buffer+1
		jsr mon_printhex
                #jmps n4
                .fi
nkn             tax
tj              lda a_last-1,x
                lsr
cccu		sta (m_p1),y
                iny
                inx
		bcc tj

n4		pla
		sta m_p3
		pla
		sta m_p3+1
		rts
                
n4i		bcc n4
                lda #"A"-64
                gcs cccu
		

phex		jsr mon_printhexsp
		inc m_p2
		bne *+4
		inc m_p2+1
		rts

mon_printopc	jsr in
		jsr *+3
		lda m_p3
		ldx #5
br		lsr m_p3+1
		ror
		dex
		bne br
in		sta m_p3
		and #31
		sta (m_p1),y
		iny
		rts
		.pend
;---------------------------
;Store a byte to drive
;
mon_storebyte_dr .proc
		tax
		lda #"W"
		jsr drive_memcmd
		lda m_p4
		jsr iecout
		lda m_p4+1
		jsr iecout
		lda #1
		jsr iecout
		txa
		jsr iecout
		jmp iecunlisten		;unlisten
		.pend
;---------------------------
;Store a byte to memory
;
mon_storebyte_mem .proc
		ldx m_p4+1
		cpx #8
		bcs ntn
		tax
		lda m_p4+1
		pha
		ora #>monitor_data_exp
		sta m_p4+1
		txa
		.if SUPERCPU=1
		sta [m_p4]
		.else
		ldx #0
		sta cfg_open
		sta (m_p4,x)
		sta cfg_16
		.fi
		pla
		sta m_p4+1
		rts

ntn		cpx #$d0
		bcc uf
		cpx #$e0
		bcs uf
		ldx m_01
		cpx #5
		bcs uf2
		ldx #3
		stx $1
		.if SUPERCPU=1
		sta (m_p4)
		.else
		ldx #0
		sta (m_p4,x)
		.fi
		lda #7
		sta $1
		rts

uf		.if SUPERCPU=1
		sta (m_p4)
		.else
		ldx #0
		sta (m_p4,x)
		.fi
		rts

uf2		#sta_cfg_open
		pha
		lda m_p4
		ldx m_p4+1
		cpx #$d4
		bcs n1
                .if SUPERCPU=1
                cpx #$d2
                .fi
		and #63
		tax
		pla
                .if SUPERCPU=1
                bcs n0
                .fi
		sta monitor_vic_exp,x
		#sta_cfg_16
n0		rts

n1		beq c64
		cpx #$d8
		bcs n2
		pha
		lda $d030		;C128?
		lsr
		pla
		bcs c64
		cpx #$d6
		bne n5
		geq vdc

c64		and #31
		tax
		pla
		sta monitor_sid_exp,x
		#sta_cfg_16
		rts

n2		cpx #$dc
		bcc color
		ora #$f0
		cpx #$dd
		beq cia2
		bcs n5
		eor #$10
cia2		tax
		pla
		sta monitor_cia1_exp-$e0,x
		#sta_cfg_16
		rts

n5		pla
		#sta_cfg_16
		rts

color		txa
		and #3
		ldx m_p4
		lsr
		bne ii3

		lda monitor_d800_exp,x
		bcs ii
		and #15
		sta monitor_d800_exp,x
		pla
		asl
		asl
		asl
		asl
ii2		ora monitor_d800_exp,x
		sta monitor_d800_exp,x
		#sta_cfg_16
		rts
ii		and #240
		sta monitor_d800_exp,x
		pla
		and #15
		gcs ii2

ii3		lda monitor_d800_exp+$100,x
		bcs ii4
		and #15
		sta monitor_d800_exp+$100,x
		pla
		asl
		asl
		asl
		asl
ii5		ora monitor_d800_exp+$100,x
		sta monitor_d800_exp+$100,x
		#sta_cfg_16
		rts
ii4		and #240
		sta monitor_d800_exp+$100,x
		pla
		and #15
		gcs ii5

vdc		and #63
		tax
		pla
		cpx #$20
		bcs oo
		cpx #$1e
		bcc oo
		pha
		#phx_a
		ldx #$12
		lda monitor_vdc_exp+$12
		jsr setcrtcreg
		inx
		lda monitor_vdc_exp+$13
		jsr setcrtcreg
		#plx_a
		pla
		jsr setcrtcreg
		#phx_a
		ldx #$12
		jsr getcrtcreg
		sta monitor_vdc_exp+$12
		inx
		jsr getcrtcreg
		sta monitor_vdc_exp+$13
		ldx #$1f
		jsr getcrtcreg
		sta monitor_vdc_exp+$1f
		pla
		cmp #$1f
		beq i2
		pha
		ldx #$20
		jsr getcrtcreg
		sta monitor_vdc_exp+$20
		inx
		jsr getcrtcreg
		sta monitor_vdc_exp+$21
		#plx_a
		#jmps i

oo		jsr setcrtcreg
i		jsr getcrtcreg
		sta monitor_vdc_exp,x
i2		lda $d600
		sta monitor_vdc_exp+63
		#sta_cfg_16
noc		rts

		.pend
;---------------------------
;Prefetch bytes from drive
;
mon_setprefetch_dr .proc
		sta m_prefetch
		lda #"R"
		jsr drive_memcmd
		lda m_p2
		jsr iecout
		lda m_p2+1
		jsr iecout
		lda m_prefetch
		cmp #1
		beq nn
		jsr iecout
nn		jsr iecunlisten		;unlisten

		lda m_01
		jsr iectalk		;talk
		lda #$6f
		jmp iectksa		;tksa
		.pend
;---------------------------
;Get a byte from internal
;
mon_getbyte_int	.proc
		.if SUPERCPU=1
		lda m_p2+1
		cmp #$a0
		bcs em
		cmp #$80
		bcs ok
		cmp #$60
		bcs em
                cmp #$10
                bcc em
ok		lda [m_p2]
		.else
		lda m_p2+1
		bmi em
		cmp #$10
		bcc em
		sta cfg_open
		ldx #0
		lda (m_p2,x)
		sta cfg_16
		.fi
		rts
em		lda #$ff
		rts
		.pend
;---------------------------
;Get a byte from VDC
;
mon_getbyte_vdc	.proc
		.if SUPERCPU=0
		lda $d030
		lsr
		bcc c128
		.fi
		lda #$ff
		rts
		.if SUPERCPU=0
c128		ldx #$13
		lda m_p2
		jsr setcrtcreg
		dex
		lda m_p2+1
		jsr setcrtcreg
		ldx #$1f
		.fi
getcrtcreg	.proc
		stx $d600
in		bit $d600
		bpl *-3
		lda $d601
		rts
		.pend
;---------------------------
;Store a byte to VDC
;
mon_storebyte_vdc .proc
		.if SUPERCPU=1
		rts
		.else
		pha
		lda $d030
		lsr
		bcs setcrtcreg.c64
		ldx #$13
		lda m_p4
		jsr setcrtcreg
		dex
		lda m_p4+1
		jsr setcrtcreg
		pla
		ldx #$1f
		.fi
setcrtcreg	.proc
		stx $d600
		bit $d600
		bpl *-3
		sta $d601
c64		rts
		.pend
;---------------------------
;Get a byte from drive
;
mon_getbyte_dr	.proc
		#stz_a Status
vsz		dec m_prefetch
		bmi one
		bne more
		jsr iecin
		tax
sux		jsr iecuntalk		;untalk
		txa
		bit Status		;to fix to m xxf1 bug
		bvc ok
one		lda #1
		jsr mon_setprefetch
		dec m_prefetch
		jsr iecin
		tax
		jsr iecuntalk		;untalk
		txa
ok		rts

more		jsr iecin
		bit Status		;to fix the I 02c0 bug
		bvs sux
		rts
		.pend
;---------------------------
;Get a byte from memory
;
mon_getbyte_mem	.proc
		ldx #0
		lda m_p2+1
		bmi uk
		cmp #8
		bcc kji
                .if SUPERCPU=1
		lda (m_p2)
                .else
		lda (m_p2,x)
                .fi
		rts

kji		.if SUPERCPU=1
		tax
		ora #>monitor_data_exp
		sta m_p2+1
		lda [m_p2]
                stx m_p2+1
		.else
		pha
		ora #>monitor_data_exp
		sta m_p2+1
		sta cfg_open
		lax (m_p2,x)
		sta cfg_16
		pla
		sta m_p2+1
		txa
		.fi
		rts

tt		lda m_01
tt2		jmp $7e8

uk		cmp #$d0
		bcc tt
		cmp #$e0
		bcs tt
		lda m_01
		cmp #5
		bcc tt2

		#sta_cfg_open
		lda m_p2
		ldx m_p2+1
		cpx #$d4
		bcs n1
		.if SUPERCPU=1
                cpx #$d2
                bcc n0
                bne n00
                tax
		lda $d200,x
		rts
n00             tax
		lda $d300,x
		rts
n0
		.fi
		and #63
		tax
		lda monitor_vic_exp,x
		#sta_cfg_16
		rts

n1		cpx #$d8
		bcs n2
		cpx #$d4
		beq c64
		pha
		lda $d030		;C128?
		lsr
		pla
		bcc nc64
c64		and #31
		tax
		lda monitor_sid_exp,x
		#sta_cfg_16
		rts

nc64		cpx #$d6
		bne n5
		and #63
		tax
		lda monitor_vdc_exp,x
		#sta_cfg_16
		rts

n2		cpx #$dc
		bcs n3
		txa
		ldx m_p2
		and #3
		lsr
		beq ct
		lda monitor_d800_exp+$100,x
		#jmps ib
ct		lda monitor_d800_exp,x
ib		bcs ed
		lsr
		lsr
		lsr
		lsr
ed		and #15
		#sta_cfg_16
		rts

n3		ora #$f0
		cpx #$dd
		beq cia2
		bcs n5
		eor #$10
cia2		tax
		lda monitor_cia1_exp-$e0,x
		.byte SKIP2
n5		lda #$ff
		#sta_cfg_16
		rts
		.pend
;---------------------------
;Convert from ascii to screencode
;
tosccode	.proc
		cmp #128
		bcs hi
		cmp #$20
		bcc inv
		cmp #$60
		bcc en
		and #$df
		bne *+5
en		and #$3f
		rts

hi		and #$7f
		cmp #$7f
		bne *+4
		lda #$5e
		cmp #$20
		ora #$40
		bcs *+4
inv		ora #$80
		rts
		.pend
;---------------------------
;Convert from screencode to ascii
;
fromsccode	.proc
		sta $d7
		and #$3f
		asl $d7
		bit $d7
		bpl *+4
		ora #$80
		bcs e
		bvs e
		ora #$40
e		rts
		.pend
;---------------------------
;Get a hex number from screen
;
gethex		.proc
		sec
		lda (m_p1),y
		beq en
		cmp #"9"+1
		bcs en
		cmp #"0"
		bcs ss
		cmp #"G"-64
		bcs en
		adc #9+"0"+1
ss		sbc #"0"
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		iny
		cpy #41
                #orasta_a m_p3
en		rts
		.pend
;---------------------------
;Get a hex address from screen
;
get_addr2x	.proc
		jsr get_addr2
		beq by
		pla
		pla
by		rts

get_addr2	.proc
		lda #32
nx		iny
		cmp (m_p1),y
		bne getaddr
		cpy #39
		bne nx
		iny
		rts

getaddr		.proc
		#stz_a m_p3
		#stz_aa m_p3+1
                .if SUPERCPU=1
		#stz_aa m_p3+2
                .fi
getaddrnc       .proc
		jsr gethex
		bcc aai
		cmp #30			;"^"?
		bne aa
		#phx_a
		jsr poptsp
		sta m_p3
		lda m_tstack+8,x
		sta m_p3+1
		#plx_a
		iny
		gcs aa
aai		jsr gethex
		bcs aa
		jsr gethex
		bcs aa
		jsr gethex
                .if SUPERCPU=1
		bcs aa
		jsr gethex
		bcs aa
		jsr gethex
                .fi
		cpy #40
		bne aa
		lda #32
		.byte SKIP2
aa		lda (m_p1),y
		cmp #32
		rts
		.pend
;---------------------------
;Peek under rom (copied $7e8)
;
mon_peek	.proc
		sta cfg_stnd		;3
		sta $1			;5
                .if SUPERCPU=1
                lda (m_p2)
                .else
		lda (m_p2,x)		;7
                .fi
		ldx #$37		;9
		stx $1			;11
		sta cfg_16		;14
		rts			;15
		.pend
;---------------------------
;Get a filename from screen
;
mon_getfname	.proc
xfn		cpy #39
		beq mon_getfnameset.err
		lda (m_p1),y
		iny
		cmp #34
		bne xfn
		ldx #0
dtn		cpy #39
		beq mon_getfnameset.err
		lda (m_p1),y
		cmp #34
		beq mon_getfnameset
		iny
		jsr fromsccode
		sta monitor_buffer,x
		inx
		bne dtn

mon_getfnameset	.proc
		stx ActualFNLength
		cpx #0
		beq err
		lda #<monitor_buffer
		sta ActualFilename
		lda #>monitor_buffer
		sta ActualFilename+1
		clc
err		rts
		.pend
;---------------------------
;Store a byte and increment pointer
;
mon_storebyte_inc .proc
		jsr mon_storebyte
		inc m_p4
		bne *+4
		inc m_p4+1
		rts
		.pend
;---------------------------
;copy m_p3+1 to m_p2+1
;
mon_copy_p3p2sbc .proc
		sta m_p2
		lda m_p3+1
		sbc #0
		sta m_p2+1
		rts
		.pend
;---------------------------
;Wait retrace
;
mon_waitretrace	.proc
		bit $d011
		bpl *-3
		bit $d011
		bmi *-3
		rts
		.pend
;---------------------------
;Clean till end of line
;
mon_clearend	.proc
		lda #32
iy		sta (m_p1),y
		iny
		cpy #40
		bcc iy
		rts
		.pend
;---------------------------
;Calc prevline
;
mon_cprevline	.proc
		lda m_p1
		sec
		sbc #40
		sta m_p1
		bcs *+4
		dec m_p1+1
		rts
		.pend
;---------------------------
;Open
;
mon_open	.proc
		sta ActualSecAddr
                
		ldy ActualDev
                jsr newclall_t		;be safe!
		sty ActualDev

		jsr OP_findunusedchannel
		jsr newopen_t
		ldx ActualLogical
		rts
		.pend
;---------------------------
;Freeze common
;
mon_frzcomm	.proc
		lda #<(monitor_freeze_exp+512)
		sta m_p2
		lda #>(monitor_freeze_exp+512)
		sta m_p2+1

		ldy #<(monitor_freeze_end_exp-monitor_freeze_exp)
		ldx #3
		rts
		.pend
;---------------------------
;Store common
;
get_addr2store	.proc
		sta m_tmp2
		jsr get_addr2
		bne brt
		lda m_p3
		sta m_p4
		sta m_p2
		lda m_p3+1
		sta m_p4+1
		sta m_p2+1
		rts
brt		pla
		pla
		rts
		.pend
;---------------------------
;get binary data from screen (for ec and es)
;
get_binary2	.proc
		ldx #8
igu		iny
		cpy #40
		beq ddb
		lda (m_p1),y
		cmp #"."
		beq e
		cmp #32
		beq ddb
		lda #1
e		lsr
		rol m_tmp
		dex
		bne igu
		lda m_tmp
		jsr mon_storebyte_inc
		clc
		.byte SKIP1
ddb		sec
		rts
		.pend
;---------------------------
;Length of addressing modes
;                

a_len		.byte 1-1+(15)*16	  		   ;A
		.byte 2-1+(a_last_xi-a_last+1)*16+4     ;($FF,x)
		.byte 3-1+(a_last_i-a_last+1)*16+4      ;($FFFF)
		.byte 2-1+(a_last_iy-a_last+1)*16+4     ;($FF),y
		.byte 3-1+(a_last_xi-a_last+1)*16+4     ;($FFFF,x)
		.byte 4-1+(a_last_x-a_last+1)*16	   ;$FFFFFF,x
		.byte 2-1+(a_last_i-a_last+1)*16+4      ;($FF)
                .byte 2-1+(a_last_siy-a_last+1)*16+4    ;($FF,s),y
                .byte 3-1+(2)*16			   ;$FF,$FF
                .byte 2-1+(a_last_il-a_last+1)*16+8     ;[$FF]
                .byte 3-1+(15)*16			   ;$FFFF
		.byte 2-1+(a_last_ily-a_last+1)*16+8    ;[$FF],y
                .byte 2-1+(15)*16	 		   ;$FFFF
                .byte 2-1+(a_last_s-a_last+1)*16	   ;$FF,s
                .byte 1-1+(0)*16                        
                .byte 1-1+(0)*16                        
                .byte 2-1+(0)*16+12                     ;#$FF
                .byte 2-1+(0)*16+12                     ;#$FF
                .byte 2-1+(0)*16			   ;$FF
                .byte 2-1+(0)*16                        ;$FF
		.byte 2-1+(a_last_x-a_last+1)*16        ;$FF,x
		.byte 2-1+(a_last_x-a_last+1)*16	   ;$FF,x
		.byte 2-1+(a_last_y-a_last+1)*16        ;$FF,y
		.byte 2-1+(a_last_y-a_last+1)*16	   ;$FF,y
		.byte 3-1+(a_last_x-a_last+1)*16        ;$FFFF,x
		.byte 3-1+(a_last_x-a_last+1)*16    	   ;$FFFF,x
		.byte 3-1+(a_last_y-a_last+1)*16        ;$FFFF,y
		.byte 3-1+(a_last_y-a_last+1)*16	   ;$FFFF,y
                .byte 3-1+(0)*16                        ;$FFFF
                .byte 3-1+(0)*16			   ;$FFFF
                .byte 4-1+(0)*16			   ;$FFFFFF
                .byte 4-1+(0)*16			   ;$FFFFFF
                
a_first         .byte "(","["-64,"#"

a_last
a_last_x       	.byte ","*2,"X"*2+1-128

a_last_xi      	.byte ","*2,"X"*2-128
a_last_i       	.byte ")"*2+1

a_last_s       	.byte ","*2,"S"*2+1-128

a_last_il      	.byte "]"*2+1-128

a_last_siy     	.byte ","*2,"S"*2-128
a_last_iy      	.byte ")"*2
a_last_y       	.byte ","*2,"Y"*2+1-128

a_last_ily     	.byte "]"*2-128,","*2,"Y"*2+1-128
;---------------------------
;Addressing modes
;
		.if SUPERCPU=1
a_mod		.byte a_imp+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;0

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpa+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abx*16
		.byte a_abx+a_lnx*16	;1

		.byte a_abs+a_zxi*16
		.byte a_lng+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;2

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_acc+a_imp*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_lnx*16	;3

		.byte a_imp+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_xya+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;4

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_xya+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_lng+a_abx*16
		.byte a_abx+a_lnx*16	;5

		.byte a_imp+a_zxi*16
		.byte a_rll+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_ind+a_abs*16
		.byte a_abs+a_lng*16	;6

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_axi+a_abx*16
		.byte a_abx+a_lnx*16	;7

		.byte a_rel+a_zxi*16
		.byte a_rll+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;8

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abx*16
		.byte a_abx+a_lnx*16	;9

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;a

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_abx+a_abx*16
		.byte a_aby+a_lnx*16	;b

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;c

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpi+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_ind+a_abx*16
		.byte a_abx+a_lnx*16	;d

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;e

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_abs+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_axi+a_abx*16
		.byte a_abx+a_lnx*16	;f
		.else
a_mod		.byte a_imp+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;0

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;1

		.byte a_abs+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;2

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;3

		.byte a_imp+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;4

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;5

		.byte a_imp+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_ind+a_abs*16
		.byte a_abs+a_abs*16	;6

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;7

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;8

		.byte a_rel+a_ziy*16
		.byte a_imp+a_abx*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zpy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_aby+a_abx*16
		.byte a_abx+a_aby*16	;9

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;a

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zpy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_aby+a_aby*16	;b

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;c

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;d

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;e

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;f
		.fi
;---------------------------
;Opcode index
;
		.if SUPERCPU=1
opc		.byte o_brk-o_table
		.byte o_ora-o_table
		.byte o_cop-o_table
		.byte o_ora-o_table
		.byte o_tsb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table
		.byte o_php-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_phd-o_table
		.byte o_tsb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table	;0

		.byte o_bpl-o_table
		.byte o_ora-o_table
		.byte o_ora-o_table
		.byte o_ora-o_table
		.byte o_trb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table
		.byte o_clc-o_table
		.byte o_ora-o_table
		.byte o_inc-o_table
		.byte o_tcs-o_table
		.byte o_trb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table	;1

		.byte o_jsr-o_table
		.byte o_and-o_table
		.byte o_jsr-o_table
		.byte o_and-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table
		.byte o_plp-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_pld-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table	;2

		.byte o_bmi-o_table
		.byte o_and-o_table
		.byte o_and-o_table
		.byte o_and-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table
		.byte o_sec-o_table
		.byte o_and-o_table
		.byte o_dec-o_table
		.byte o_tsc-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table	;3

		.byte o_rti-o_table
		.byte o_eor-o_table
		.byte o_wdm-o_table
		.byte o_eor-o_table
		.byte o_mvp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table
		.byte o_pha-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_phk-o_table
		.byte o_jmp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table	;4

		.byte o_bvc-o_table
		.byte o_eor-o_table
		.byte o_eor-o_table
		.byte o_eor-o_table
		.byte o_mvn-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table
		.byte o_cli-o_table
		.byte o_eor-o_table
		.byte o_phy-o_table
		.byte o_tcd-o_table
		.byte o_jmp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table	;5

		.byte o_rts-o_table
		.byte o_adc-o_table
		.byte o_per-o_table
		.byte o_adc-o_table
		.byte o_stz-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table
		.byte o_pla-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rtl-o_table
		.byte o_jmp-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table	;6

		.byte o_bvs-o_table
		.byte o_adc-o_table
		.byte o_adc-o_table
		.byte o_adc-o_table
		.byte o_stz-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table
		.byte o_sei-o_table
		.byte o_adc-o_table
		.byte o_ply-o_table
		.byte o_tdc-o_table
		.byte o_jmp-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table	;7

		.byte o_bra-o_table
		.byte o_sta-o_table
		.byte o_brl-o_table
		.byte o_sta-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sta-o_table
		.byte o_dey-o_table
		.byte o_bit-o_table
		.byte o_txa-o_table
		.byte o_phb-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sta-o_table	;8

		.byte o_bcc-o_table
		.byte o_sta-o_table
		.byte o_sta-o_table
		.byte o_sta-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sta-o_table
		.byte o_tya-o_table
		.byte o_sta-o_table
		.byte o_txs-o_table
		.byte o_txy-o_table
		.byte o_stz-o_table
		.byte o_sta-o_table
		.byte o_stz-o_table
		.byte o_sta-o_table	;9

		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table
		.byte o_tay-o_table
		.byte o_lda-o_table
		.byte o_tax-o_table
		.byte o_plb-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table	;a

		.byte o_bcs-o_table
		.byte o_lda-o_table
		.byte o_lda-o_table
		.byte o_lda-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table
		.byte o_clv-o_table
		.byte o_lda-o_table
		.byte o_tsx-o_table
		.byte o_tyx-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table	;b

		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_rep-o_table
		.byte o_cmp-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table
		.byte o_iny-o_table
		.byte o_cmp-o_table
		.byte o_dex-o_table
		.byte o_wai-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table	;c

		.byte o_bne-o_table
		.byte o_cmp-o_table
		.byte o_cmp-o_table
		.byte o_cmp-o_table
		.byte o_pei-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table
		.byte o_cld-o_table
		.byte o_cmp-o_table
		.byte o_phx-o_table
		.byte o_stp-o_table
		.byte o_jmp-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table	;d

		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_sep-o_table
		.byte o_sbc-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table
		.byte o_inx-o_table
		.byte o_sbc-o_table
		.byte o_nop-o_table
		.byte o_xba-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table	;e

		.byte o_beq-o_table
		.byte o_sbc-o_table
		.byte o_sbc-o_table
		.byte o_sbc-o_table
		.byte o_pea-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table
		.byte o_sed-o_table
		.byte o_sbc-o_table
		.byte o_plx-o_table
		.byte o_xce-o_table
		.byte o_jsr-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table	;f
		.else                
opc		.byte o_brk-o_table
		.byte o_ora-o_table
		.byte o_jam-o_table
		.byte o_slo-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table
		.byte o_php-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_anc-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table	;0

		.byte o_bpl-o_table
		.byte o_ora-o_table
		.byte o_jam-o_table
		.byte o_slo-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table
		.byte o_clc-o_table
		.byte o_ora-o_table
		.byte o_no2-o_table
		.byte o_slo-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table	;1

		.byte o_jsr-o_table
		.byte o_and-o_table
		.byte o_jam-o_table
		.byte o_rla-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table
		.byte o_plp-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_anc-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table	;2

		.byte o_bmi-o_table
		.byte o_and-o_table
		.byte o_jam-o_table
		.byte o_rla-o_table
		.byte o_no2-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table
		.byte o_sec-o_table
		.byte o_and-o_table
		.byte o_no2-o_table
		.byte o_rla-o_table
		.byte o_no2-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table	;3

		.byte o_rti-o_table
		.byte o_eor-o_table
		.byte o_jam-o_table
		.byte o_sre-o_table
		.byte o_no2-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table
		.byte o_pha-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_asr-o_table
		.byte o_jmp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table	;4

		.byte o_bvc-o_table
		.byte o_eor-o_table
		.byte o_jam-o_table
		.byte o_sre-o_table
		.byte o_no2-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table
		.byte o_cli-o_table
		.byte o_eor-o_table
		.byte o_no2-o_table
		.byte o_sre-o_table
		.byte o_no2-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table	;5

		.byte o_rts-o_table
		.byte o_adc-o_table
		.byte o_jam-o_table
		.byte o_rra-o_table
		.byte o_no2-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table
		.byte o_pla-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_arr-o_table
		.byte o_jmp-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table	;6

		.byte o_bvs-o_table
		.byte o_adc-o_table
		.byte o_jam-o_table
		.byte o_rra-o_table
		.byte o_no2-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table
		.byte o_sei-o_table
		.byte o_adc-o_table
		.byte o_no2-o_table
		.byte o_rra-o_table
		.byte o_no2-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table	;7

		.byte o_no2-o_table
		.byte o_sta-o_table
		.byte o_no2-o_table
		.byte o_sax-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sax-o_table
		.byte o_dey-o_table
		.byte o_no2-o_table
		.byte o_txa-o_table
		.byte o_ane-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sax-o_table	;8

		.byte o_bcc-o_table
		.byte o_sta-o_table
		.byte o_jam-o_table
		.byte o_sha-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_sax-o_table
		.byte o_stx-o_table
		.byte o_tya-o_table
		.byte o_sta-o_table
		.byte o_txs-o_table
		.byte o_shs-o_table
		.byte o_shy-o_table
		.byte o_sta-o_table
		.byte o_shx-o_table
		.byte o_sha-o_table	;9

		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table
		.byte o_tay-o_table
		.byte o_lda-o_table
		.byte o_tax-o_table
		.byte o_lxa-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table	;a

		.byte o_bcs-o_table
		.byte o_lda-o_table
		.byte o_jam-o_table
		.byte o_lax-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table
		.byte o_clv-o_table
		.byte o_lda-o_table
		.byte o_tsx-o_table
		.byte o_lae-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table	;b

		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_no2-o_table
		.byte o_dcp-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table
		.byte o_iny-o_table
		.byte o_cmp-o_table
		.byte o_dex-o_table
		.byte o_sbx-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table	;c

		.byte o_bne-o_table
		.byte o_cmp-o_table
		.byte o_jam-o_table
		.byte o_dcp-o_table
		.byte o_no2-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table
		.byte o_cld-o_table
		.byte o_cmp-o_table
		.byte o_no2-o_table
		.byte o_dcp-o_table
		.byte o_no2-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table	;d

		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_no2-o_table
		.byte o_isb-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table
		.byte o_inx-o_table
		.byte o_sbc-o_table
		.byte o_nop-o_table
		.byte o_sb2-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table	;e

		.byte o_beq-o_table
		.byte o_sbc-o_table
		.byte o_jam-o_table
		.byte o_isb-o_table
		.byte o_no2-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table
		.byte o_sed-o_table
		.byte o_sbc-o_table
		.byte o_no2-o_table
		.byte o_isb-o_table
		.byte o_no2-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table	;f
                .fi
;---------------------------
;Opcodes
;
		.if SUPERCPU=1
o_table
o_adc		.word ("A" & 31)+("D" & 31)*32+("C" & 31)*1024
o_and		.word ("A" & 31)+("N" & 31)*32+("D" & 31)*1024
o_asl		.word ("A" & 31)+("S" & 31)*32+("L" & 31)*1024
o_bcc		.word ("B" & 31)+("C" & 31)*32+("C" & 31)*1024
o_bcs		.word ("B" & 31)+("C" & 31)*32+("S" & 31)*1024
o_beq		.word ("B" & 31)+("E" & 31)*32+("Q" & 31)*1024
o_bit		.word ("B" & 31)+("I" & 31)*32+("T" & 31)*1024
o_bmi		.word ("B" & 31)+("M" & 31)*32+("I" & 31)*1024
o_bne		.word ("B" & 31)+("N" & 31)*32+("E" & 31)*1024
o_bpl		.word ("B" & 31)+("P" & 31)*32+("L" & 31)*1024

o_bra		.word ("B" & 31)+("R" & 31)*32+("A" & 31)*1024
o_brk		.word ("B" & 31)+("R" & 31)*32+("K" & 31)*1024
o_brl		.word ("B" & 31)+("R" & 31)*32+("L" & 31)*1024
o_bvc		.word ("B" & 31)+("V" & 31)*32+("C" & 31)*1024
o_bvs		.word ("B" & 31)+("V" & 31)*32+("S" & 31)*1024
o_clc		.word ("C" & 31)+("L" & 31)*32+("C" & 31)*1024
o_cld		.word ("C" & 31)+("L" & 31)*32+("D" & 31)*1024
o_cli		.word ("C" & 31)+("L" & 31)*32+("I" & 31)*1024
o_clv		.word ("C" & 31)+("L" & 31)*32+("V" & 31)*1024
o_cmp		.word ("C" & 31)+("M" & 31)*32+("P" & 31)*1024

o_cop		.word ("C" & 31)+("O" & 31)*32+("P" & 31)*1024
o_cpx		.word ("C" & 31)+("P" & 31)*32+("X" & 31)*1024
o_cpy		.word ("C" & 31)+("P" & 31)*32+("Y" & 31)*1024
o_dec		.word ("D" & 31)+("E" & 31)*32+("C" & 31)*1024
o_dex		.word ("D" & 31)+("E" & 31)*32+("X" & 31)*1024
o_dey		.word ("D" & 31)+("E" & 31)*32+("Y" & 31)*1024
o_eor		.word ("E" & 31)+("O" & 31)*32+("R" & 31)*1024
o_inc		.word ("I" & 31)+("N" & 31)*32+("C" & 31)*1024
o_inx		.word ("I" & 31)+("N" & 31)*32+("X" & 31)*1024
o_iny		.word ("I" & 31)+("N" & 31)*32+("Y" & 31)*1024

o_jmp		.word ("J" & 31)+("M" & 31)*32+("P" & 31)*1024
o_jsr		.word ("J" & 31)+("S" & 31)*32+("R" & 31)*1024
o_lda		.word ("L" & 31)+("D" & 31)*32+("A" & 31)*1024
o_ldx		.word ("L" & 31)+("D" & 31)*32+("X" & 31)*1024
o_ldy		.word ("L" & 31)+("D" & 31)*32+("Y" & 31)*1024
o_lsr		.word ("L" & 31)+("S" & 31)*32+("R" & 31)*1024
o_mvn		.word ("M" & 31)+("V" & 31)*32+("N" & 31)*1024
o_mvp		.word ("M" & 31)+("V" & 31)*32+("P" & 31)*1024
o_nop		.word ("N" & 31)+("O" & 31)*32+("P" & 31)*1024
o_ora		.word ("O" & 31)+("R" & 31)*32+("A" & 31)*1024

o_pea		.word ("P" & 31)+("E" & 31)*32+("A" & 31)*1024
o_pei		.word ("P" & 31)+("E" & 31)*32+("I" & 31)*1024
o_per		.word ("P" & 31)+("E" & 31)*32+("R" & 31)*1024
o_pha		.word ("P" & 31)+("H" & 31)*32+("A" & 31)*1024
o_phb		.word ("P" & 31)+("H" & 31)*32+("B" & 31)*1024
o_phd		.word ("P" & 31)+("H" & 31)*32+("D" & 31)*1024
o_phk		.word ("P" & 31)+("H" & 31)*32+("K" & 31)*1024
o_php		.word ("P" & 31)+("H" & 31)*32+("P" & 31)*1024
o_phx		.word ("P" & 31)+("H" & 31)*32+("X" & 31)*1024
o_phy		.word ("P" & 31)+("H" & 31)*32+("Y" & 31)*1024

o_pla		.word ("P" & 31)+("L" & 31)*32+("A" & 31)*1024
o_plb		.word ("P" & 31)+("L" & 31)*32+("B" & 31)*1024
o_pld		.word ("P" & 31)+("L" & 31)*32+("D" & 31)*1024
o_plp		.word ("P" & 31)+("L" & 31)*32+("P" & 31)*1024
o_plx   	.word ("P" & 31)+("L" & 31)*32+("X" & 31)*1024
o_ply   	.word ("P" & 31)+("L" & 31)*32+("Y" & 31)*1024
o_rep   	.word ("R" & 31)+("E" & 31)*32+("P" & 31)*1024
o_rol		.word ("R" & 31)+("O" & 31)*32+("L" & 31)*1024
o_ror		.word ("R" & 31)+("O" & 31)*32+("R" & 31)*1024
o_rti		.word ("R" & 31)+("T" & 31)*32+("I" & 31)*1024

o_rtl		.word ("R" & 31)+("T" & 31)*32+("L" & 31)*1024
o_rts		.word ("R" & 31)+("T" & 31)*32+("S" & 31)*1024
o_sbc		.word ("S" & 31)+("B" & 31)*32+("C" & 31)*1024
o_sec		.word ("S" & 31)+("E" & 31)*32+("C" & 31)*1024
o_sed		.word ("S" & 31)+("E" & 31)*32+("D" & 31)*1024
o_sei		.word ("S" & 31)+("E" & 31)*32+("I" & 31)*1024
o_sep		.word ("S" & 31)+("E" & 31)*32+("P" & 31)*1024
o_sta		.word ("S" & 31)+("T" & 31)*32+("A" & 31)*1024
o_stp		.word ("S" & 31)+("T" & 31)*32+("P" & 31)*1024
o_stx		.word ("S" & 31)+("T" & 31)*32+("X" & 31)*1024

o_sty		.word ("S" & 31)+("T" & 31)*32+("Y" & 31)*1024
o_stz		.word ("S" & 31)+("T" & 31)*32+("Z" & 31)*1024
o_tax		.word ("T" & 31)+("A" & 31)*32+("X" & 31)*1024
o_tay		.word ("T" & 31)+("A" & 31)*32+("Y" & 31)*1024
o_tcd		.word ("T" & 31)+("C" & 31)*32+("D" & 31)*1024
o_tcs		.word ("T" & 31)+("C" & 31)*32+("S" & 31)*1024
o_tdc		.word ("T" & 31)+("D" & 31)*32+("C" & 31)*1024
o_trb		.word ("T" & 31)+("R" & 31)*32+("B" & 31)*1024
o_tsb		.word ("T" & 31)+("S" & 31)*32+("B" & 31)*1024
o_tsc		.word ("T" & 31)+("S" & 31)*32+("C" & 31)*1024

o_tsx		.word ("T" & 31)+("S" & 31)*32+("X" & 31)*1024
o_txa		.word ("T" & 31)+("X" & 31)*32+("A" & 31)*1024
o_txs		.word ("T" & 31)+("X" & 31)*32+("S" & 31)*1024
o_txy		.word ("T" & 31)+("X" & 31)*32+("Y" & 31)*1024
o_tya		.word ("T" & 31)+("Y" & 31)*32+("A" & 31)*1024
o_tyx		.word ("T" & 31)+("Y" & 31)*32+("X" & 31)*1024
o_wai		.word ("W" & 31)+("A" & 31)*32+("I" & 31)*1024
o_wdm		.word ("W" & 31)+("D" & 31)*32+("M" & 31)*1024+32768
o_xba		.word ("X" & 31)+("B" & 31)*32+("A" & 31)*1024
o_xce		.word ("X" & 31)+("C" & 31)*32+("E" & 31)*1024
                .else
o_table
o_adc		.word ("A" & 31)+("D" & 31)*32+("C" & 31)*1024
o_anc		.word ("A" & 31)+("N" & 31)*32+("C" & 31)*1024+32768
o_and		.word ("A" & 31)+("N" & 31)*32+("D" & 31)*1024
o_ane		.word ("A" & 31)+("N" & 31)*32+("E" & 31)*1024+32768
o_arr		.word ("A" & 31)+("R" & 31)*32+("R" & 31)*1024+32768
o_asl		.word ("A" & 31)+("S" & 31)*32+("L" & 31)*1024
o_asr		.word ("A" & 31)+("S" & 31)*32+("R" & 31)*1024+32768
o_bcc		.word ("B" & 31)+("C" & 31)*32+("C" & 31)*1024
o_bcs		.word ("B" & 31)+("C" & 31)*32+("S" & 31)*1024
o_beq		.word ("B" & 31)+("E" & 31)*32+("Q" & 31)*1024

o_bit		.word ("B" & 31)+("I" & 31)*32+("T" & 31)*1024
o_bmi		.word ("B" & 31)+("M" & 31)*32+("I" & 31)*1024
o_bne		.word ("B" & 31)+("N" & 31)*32+("E" & 31)*1024
o_bpl		.word ("B" & 31)+("P" & 31)*32+("L" & 31)*1024
o_brk		.word ("B" & 31)+("R" & 31)*32+("K" & 31)*1024
o_bvc		.word ("B" & 31)+("V" & 31)*32+("C" & 31)*1024
o_bvs		.word ("B" & 31)+("V" & 31)*32+("S" & 31)*1024
o_clc		.word ("C" & 31)+("L" & 31)*32+("C" & 31)*1024
o_cld		.word ("C" & 31)+("L" & 31)*32+("D" & 31)*1024
o_cli		.word ("C" & 31)+("L" & 31)*32+("I" & 31)*1024

o_clv		.word ("C" & 31)+("L" & 31)*32+("V" & 31)*1024
o_cmp		.word ("C" & 31)+("M" & 31)*32+("P" & 31)*1024
o_cpx		.word ("C" & 31)+("P" & 31)*32+("X" & 31)*1024
o_cpy		.word ("C" & 31)+("P" & 31)*32+("Y" & 31)*1024
o_dcp		.word ("D" & 31)+("C" & 31)*32+("P" & 31)*1024+32768
o_dec		.word ("D" & 31)+("E" & 31)*32+("C" & 31)*1024
o_dex		.word ("D" & 31)+("E" & 31)*32+("X" & 31)*1024
o_dey		.word ("D" & 31)+("E" & 31)*32+("Y" & 31)*1024
o_eor		.word ("E" & 31)+("O" & 31)*32+("R" & 31)*1024
o_inc		.word ("I" & 31)+("N" & 31)*32+("C" & 31)*1024

o_inx		.word ("I" & 31)+("N" & 31)*32+("X" & 31)*1024
o_iny		.word ("I" & 31)+("N" & 31)*32+("Y" & 31)*1024
o_isb		.word ("I" & 31)+("S" & 31)*32+("B" & 31)*1024+32768
o_jam		.word ("J" & 31)+("A" & 31)*32+("M" & 31)*1024+32768
o_jmp		.word ("J" & 31)+("M" & 31)*32+("P" & 31)*1024
o_jsr		.word ("J" & 31)+("S" & 31)*32+("R" & 31)*1024
o_lae		.word ("L" & 31)+("A" & 31)*32+("E" & 31)*1024+32768
o_lax		.word ("L" & 31)+("A" & 31)*32+("X" & 31)*1024+32768
o_lda		.word ("L" & 31)+("D" & 31)*32+("A" & 31)*1024
o_ldx		.word ("L" & 31)+("D" & 31)*32+("X" & 31)*1024

o_ldy		.word ("L" & 31)+("D" & 31)*32+("Y" & 31)*1024
o_lsr		.word ("L" & 31)+("S" & 31)*32+("R" & 31)*1024
o_lxa		.word ("L" & 31)+("X" & 31)*32+("A" & 31)*1024+32768
o_nop		.word ("N" & 31)+("O" & 31)*32+("P" & 31)*1024
o_no2		.word ("N" & 31)+("O" & 31)*32+("P" & 31)*1024+32768
o_ora		.word ("O" & 31)+("R" & 31)*32+("A" & 31)*1024
o_pha		.word ("P" & 31)+("H" & 31)*32+("A" & 31)*1024
o_php		.word ("P" & 31)+("H" & 31)*32+("P" & 31)*1024
o_pla		.word ("P" & 31)+("L" & 31)*32+("A" & 31)*1024
o_plp		.word ("P" & 31)+("L" & 31)*32+("P" & 31)*1024

o_rla		.word ("R" & 31)+("L" & 31)*32+("A" & 31)*1024+32768
o_rol		.word ("R" & 31)+("O" & 31)*32+("L" & 31)*1024
o_ror		.word ("R" & 31)+("O" & 31)*32+("R" & 31)*1024
o_rra		.word ("R" & 31)+("R" & 31)*32+("A" & 31)*1024+32768
o_rti		.word ("R" & 31)+("T" & 31)*32+("I" & 31)*1024
o_rts		.word ("R" & 31)+("T" & 31)*32+("S" & 31)*1024
o_sax		.word ("S" & 31)+("A" & 31)*32+("X" & 31)*1024+32768
o_sbc		.word ("S" & 31)+("B" & 31)*32+("C" & 31)*1024
o_sb2		.word ("S" & 31)+("B" & 31)*32+("C" & 31)*1024+32768
o_sbx		.word ("S" & 31)+("B" & 31)*32+("X" & 31)*1024+32768

o_sec		.word ("S" & 31)+("E" & 31)*32+("C" & 31)*1024
o_sed		.word ("S" & 31)+("E" & 31)*32+("D" & 31)*1024
o_sei		.word ("S" & 31)+("E" & 31)*32+("I" & 31)*1024
o_sha		.word ("S" & 31)+("H" & 31)*32+("A" & 31)*1024+32768
o_shs		.word ("S" & 31)+("H" & 31)*32+("S" & 31)*1024+32768
o_shx		.word ("S" & 31)+("H" & 31)*32+("X" & 31)*1024+32768
o_shy		.word ("S" & 31)+("H" & 31)*32+("Y" & 31)*1024+32768
o_slo		.word ("S" & 31)+("L" & 31)*32+("O" & 31)*1024+32768
o_sre		.word ("S" & 31)+("R" & 31)*32+("E" & 31)*1024+32768
o_sta		.word ("S" & 31)+("T" & 31)*32+("A" & 31)*1024

o_stx		.word ("S" & 31)+("T" & 31)*32+("X" & 31)*1024
o_sty		.word ("S" & 31)+("T" & 31)*32+("Y" & 31)*1024
o_tax		.word ("T" & 31)+("A" & 31)*32+("X" & 31)*1024
o_tay		.word ("T" & 31)+("A" & 31)*32+("Y" & 31)*1024
o_tsx		.word ("T" & 31)+("S" & 31)*32+("X" & 31)*1024
o_txa		.word ("T" & 31)+("X" & 31)*32+("A" & 31)*1024
o_txs		.word ("T" & 31)+("X" & 31)*32+("S" & 31)*1024
o_tya		.word ("T" & 31)+("Y" & 31)*32+("A" & 31)*1024
		.fi
;---------------------------
;Normal and BRK start text
;
txtmon		.byte "I"-64,"D"-64,"E"-64,"6","4"," ","M"-64,"O"-64,"N"-64,"I"-64,"T"-64,"O"-64,"R"-64
txtbrk		.byte "B"-64,"R"-64,"K"-64," ","E"-64,"X"-64,"C"-64,"E"-64,"P"-64,"T"-64,"I"-64,"O"-64,"N"-64
