;
; trap.asm - IO space extension
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

		.if BANK !=0

nmi_IDE_16	.proc
		cld
		.if DE32_EXISTS=1
		and #3
		.fi
		sta nmi_oldconfig_exp
		#phx_a
		#phy_a

		ldy $dd0d		;Test if CIA#2 called this MNI
		bpl nmi_no_cia
		lda #$7f
		sta $dd0d
		ldy $dd0d
nmi_into_rom	#psr $fe72

		.if DE32_EXISTS=1
		#ldatax nmi_oldconfig_exp
		lsr
		#lda0
		sta nmi_oldconfig_exp	;drop oldconfig
		inc $318
		bcs switch_norm
		txa
		lsr
		bcc switch_16k
		jmp switch_8k
		.else
		lda nmi_oldconfig_exp
		cmp #$ff
		#lda0
		sta nmi_oldconfig_exp	;drop oldconfig
		inc $318
		bcs switch_16k
		jmp switch_norm
		.fi

nmi_no_cia	lda $dc02
                pha
                lda $dc03
                ldx #$ff
                stx $dc02
                .if SUPERCPU=1
                stz $dc03
                .else
		inx
                stx $dc03
                .fi
                ldy $dc00
                ldx #$7f
                stx $dc00

		ldx $dc01		;check stop key
		cpx $dc01
		bne *-6

                sty $dc00
                sta $dc03
                pla
                sta $dc02

		.if BANK=3
		lda nmi_oldconfig_exp
		.if DE32_EXISTS=1
		cmp #3
		bcc ddp
		.else
		cmp #$ff
		bcs ddp
		.fi			;DE32_EXISTS
		cpx #%11011111
		bne ddp
                .if SUPERCPU=0
		txa
		tsx
		inx			;y
		inx			;x
		inx			;a
		inx			;sr
		inx			;l
		inx			;h
		ldy $100,x
		tax
		cpy #>nmi_return
		beq ddp
                .fi			;SUPERCPU=0
		lda zp_valid_exp
		bne ddp
		lda #BANK
		jmp monitor
ddp
		.fi			;BANK=3

		txa
		bmi ht
		ldx #$bd
		stx $dc00
		ldx $dc01
		cpx $dc01
		bne *-6
		sta $dc00
		inx
		bne ht2
ht		sta $91
ht2
		lda $91
		eor #$7f		;TEST "RUN/STOP"
		bne nmi_into_rom

		.if DE32_EXISTS=1
		#psr $a000
		.else
		#psr $a001
		.fi
		jmp gotobank0
		.pend

		.fi			;Bank !=0



		.if BANK=0
syscall		.proc
		.if SUPERCPU=1
		sec
		xce
		bcc syscnotfound	;Yo lamer!
		xba
		.else
		sta help_trap_accu_exp
		.fi
		#stx_l_a help_x_exp
		ldx #sysend-sysstart-1
		pla
hthe		cmp sysstart,x
		beq syscfound
		dex
		bpl hthe
		pla
syscnotfound	sec
		jmp exit2basic		;Someone playing games...

syscfound	pla
		cmp #$de
		bne syscnotfound
		lda syshi,x
		pha
		lda syslo,x
		pha
		#ldatax help_x_exp
		.if SUPERCPU=1
		xba
		.else
		lda help_trap_accu_exp
		.fi
		rts

monitor2	
		.if SUPERCPU=1
		sta scpu_hwdisable
                .fi
		lda #BANK
		jmp monitor

sysstart	.byte <(r_brk+2)
		.byte <(r_mytoken+2)
		.byte <(r_clall+2)
		.byte <(r_clrchn+2)
		.byte <(r_load+2)
		.byte <(r_save+2)
		.byte <(r_open+2)
		.byte <(r_close+2)
		.byte <(r_chkin+2)
		.byte <(r_chkout+2)
		.byte <(write_dt+2)
		.byte <(read_dt+2)
sysend
syslo		.byte <(monitor2-1)
		.byte <(LW_mytoken-1)
		.byte <(newclall-1)
		.byte <(newclrchn-1)
		.byte <(newload-1)
		.byte <(newsave-1)
		.byte <(newopen-1)
		.byte <(newclose-1)
		.byte <(newchkin-1)
		.byte <(newchkout-1)
		.byte <(newwrite-1)
		.byte <(newread-1)

syshi		.byte >(monitor2-1)
		.byte >(LW_mytoken-1)
		.byte >(newclall-1)
		.byte >(newclrchn-1)
		.byte >(newload-1)
		.byte >(newsave-1)
		.byte >(newopen-1)
		.byte >(newclose-1)
		.byte >(newchkin-1)
		.byte >(newchkout-1)
		.byte >(newwrite-1)
		.byte >(newread-1)
		.pend
		.fi			;Bank=0



		.if BANK=0
;-----------------------------------------------------------
; OPEN
;
newopen_t	.proc
		#psr switch_16k
		jmp r_open
		.pend

;-----------------------------------------------------------
; CLOSE
;
newclose_t	.proc
		.if SUPERCPU=0
		tax
		.fi			;SUPERCPU=0
		#psr switch_16k
		.if SUPERCPU=0
		txa
		.fi			;SUPERCPU=0
		jmp r_close
		.pend

;-----------------------------------------------------------
; CHKIN
;
newchkin_t	.proc
		#psr switch_16k
		jmp r_chkin
		.pend

;-----------------------------------------------------------
; CLRCHN
;
newclrchn_t	.proc
		#psr switch_16k
		jmp r_clrchn
		.pend

		.fi			; BANK=0
;-----------------------------------------------------------
; LOAD
;
		.if BANK=2
newload_t	.proc
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		#psr $ffd5
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		stx $c3
		sty $c4
		jmp exit2basic
		.pend
		.fi			; BANK=2
;-----------------------------------------------------------
; READ
;
		.if (BANK=3) | (BANK=2)
newread_t	.proc
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		#psr $def4
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp exit2basic
		.pend
		.fi			; BANK=3

		.if (BANK=2)
newwrite_t	.proc
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp write_dt
		.pend
		.fi			; BANK=2

		.if (BANK=0)
newwrite_t	.proc
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr switch_16k
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp write_dt
		.pend
		.fi			; BANK=0

		.if (BANK=3)
newwrite_t_mon	.proc
		dec $1
                jsr fs
                inc $1
                rts
fs
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp write_dt
		.pend
		.fi			; BANK=3
;-----------------------------------------------------------
; OPEN
;
		.if BANK=2
newopen_t	.proc
		#psr gotobank0back
		#psr $ffc0
		jmp exit2basic
		.pend
		.fi			; BANK=2

		.if (BANK=3) | (BANK=1)
newopen_t	.proc
		#psr gotobank0back
		jmp r_open
		.pend
		.fi			; BANK=3

;-----------------------------------------------------------
; CLALL
;
newclall_t	.if BANK=2
		.proc
		#psr gotobank0back
		#psr $ffe7
		jmp exit2basic
		.pend
                .fi			; BANK=2
;-----------------------------------------------------------
; CLOSE
;
		.if BANK=2
newclose_t	.proc
		.if SUPERCPU=0
		tax
		.fi
		#psr gotobank0back
		#psr $ffc3
		.if SUPERCPU=0
		txa
		.fi
		jmp exit2basic
		.pend
		.fi			; BANK=2

		.if (BANK=3) | (BANK=1)
newclose_t	.proc
		.if SUPERCPU=0
		tax
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		txa
		.fi
		jmp r_close
		.pend
		.fi			; BANK=3

;-----------------------------------------------------------
; CHKIN
;
		.if BANK=2
newchkin_t	.proc
		#psr gotobank0back
		#psr $ffc6
		jmp exit2basic
		.pend
		.fi			; BANK=2

		.if BANK=3
newchkin_t	.proc
		#psr gotobank0back
		jmp r_chkin
		.pend
		.fi			; BANK=3
;-----------------------------------------------------------
; CHKOUT
;
		.if BANK=2
newchkout_t	.proc
		#psr gotobank0back
		#psr $ffc9
		jmp exit2basic
		.pend
		.fi			; BANK=2

		.if (BANK=3) | (BANK=1)
newchkout_t	.proc
		#psr gotobank0back
		jmp r_chkout
		.pend
		.fi			; BANK=3

		.if BANK=0
newchkout_t	.proc
		#psr switch_16k
		jmp r_chkout
		.pend
		.fi			; BANK=2

;-----------------------------------------------------------
; CLRCHN
;
		.if (BANK=2) | (BANK=3)
newclrchn_t	.proc
		#psr gotobank0back
		#psr $ffcc
		jmp exit2basic
		.pend
		.fi			; BANK=2 | 3

		.if (BANK=1)
newclrchn_t	.proc
		#psr gotobank0back
		jmp r_clrchn
		.pend
		.fi			; BANK=1


newchrin_t	.if BANK=0
		#psr switch_16k
		.else
		#psr gotobank0back
		.fi			; BANK=0
		jmp r_chrin

newgetin_t	.if BANK=0
		#psr switch_16k
		.else
		#psr gotobank0back
		#psr exit2basic
		.fi			; BANK=0
		jmp r_getin

		.if BANK!=0
newchrout_t	sta cfg_open
		.if SUPERCPU=1
		xba
		.else
		sta help_trap_accu_exp
		.fi

		lda buffer_output_exp
		eor #STDOUT
		bne ueo
		lda buffer_input_exp
		cmp #STDIN
		bne ueo
		.if BANK=3
		#psr switch_16k
		.else
		#psr gotobank3back
		.fi			; BANK=3
		gcs ueo2

ueo		#psr gotobank0back
ueo2
		.if SUPERCPU=1
		xba
		.else
		lda help_trap_accu_exp
		.fi
		jmp r_chrout
		.fi			; BANK!=0


		.if BANK=0
newload_t	.proc
		#sta_cfg_open
                .if SUPERCPU=0
                sta help_trap_accu_exp
                .fi
		#psr switch_16k
                .if SUPERCPU=0
                lda help_trap_accu_exp
                .fi
		stx $c3
		sty $c4
		jmp r_load
		.pend
		.fi			; BANK=2

;----------------------------------
; System vectors
;
		.if SUPERCPU=0

		.if *>$bffa
		.error "Overflow in range $a000-$bff9!"
		.fi

		*= $bffa		;$fffa
		.logical $fffa
		.word opencfg_nmi	;NMI
debugzp		brk
		brk
		.word opencfg_irqbrk	;IRQ/BRK
		.here

		.else			;SuperCPU

endofbank
        	.if *>$c000
;       	.error "Overflow in range $a000-$bfff!"
        	.fi

		.fi			;SuperCPU
;----------------------------------
; $DE60-$DEFF stuff
;
		*= $9e60
		.logical $de60
		.text "IDE"		;Soci: A way of detecting IDE64:

		.if BANK=0
		sta bank_port0
switch_norm
exit2basic	sta cfg_stnd		;$DE60="IDE"
		rts

		.else
exit2basic	sta bank_port0
switch_norm	sta cfg_stnd
		rts
		.fi			; BANK=0

		.if SUPERCPU=0
irq_call	sta cfg_stnd
		jmp ($fffe)

nmi_call	sta cfg_stnd
		jmp ($fffa)
                .fi

;------------------------------------------------------------
; Crossbank jmp
;
		.if BANK=0
gotobank1back	= *
gotobank2back	= *+3
gotobank3back	= *+6
gotobank3	sta bank_port1
gotobank2	sta bank_port1
gotobank1	sta bank_port1
switch_16k	
		sta cfg_16
		rts
		.fi			; BANK=0
;------------------
		.if BANK=1
gotobank2back	= *
gotobank3back	= *+3
gotobank0back	= *+6
gotobank0	sta bank_port2
gotobank3	sta bank_port2
gotobank2	sta bank_port2
switch_16k	
		sta cfg_16
		rts
		.fi			; BANK=1
;------------------
		.if BANK=2
gotobank3back	= *
gotobank0back	= *+3
gotobank1back	= *+6
gotobank1	sta bank_port3
gotobank0	sta bank_port3
gotobank3	sta bank_port3
switch_16k	
		sta cfg_16
		rts
		.fi			; BANK=2
;------------------
		.if BANK=3
gotobank0back	= *
gotobank1back	= *+3
gotobank2back	= *+6
gotobank2	sta bank_port0
gotobank1	sta bank_port0
gotobank0	sta bank_port0
switch_16k	
		sta cfg_16
		rts
		.fi			; BANK=3

r_mytoken	jsr sysc

nmi_return	pla
irq_return	sta cfg_open
block_nmi	rti			;must be right before!
r_nmi		dec $318
		pha
					;Soci: Solved the cfg_16 vs. cfg_stnd
					;      problem!
		.if DE32_EXISTS=1
		lda ideconf_port	;Undocumented register?!
		.else
		lda config16test
		.fi
		sta cfg_open

		.if BANK=0
		jmp restore
		.else
		jmp nmi_IDE_16
		.fi
;------------------------------------------------------------
; System calls from vector table
;
r_brk		jsr sysc
r_load		jsr sysc
r_save		jsr sysc
r_open		jsr sysc
r_close		jsr sysc
r_chkin		jsr sysc
r_chkout	jsr sysc
r_clall		jsr sysc
r_clrchn	jsr sysc

;--------------------------------------
switch_8k0
		.if BANK=0
switch_8k	sta cfg_8
		rts
r_chrin		sta cfg_open		;$3xx
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jmp newchrin_ram_exp
		.else
		jmp newchrin
		.fi
r_chrout	sta cfg_open		;$3xx
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jmp newchrout_ram_exp
		.else
		jmp newchrout
		.fi
r_getin		sta cfg_open		;$3xx
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jmp newgetin_ram_exp
		.else
		jmp newgetin
		.fi
		.byte $ff
		.fi
;--------------------------------------
		.if BANK=1
		.byte $ff
r_chrin		sta bank_port0
		jmp newchrin_t		;$3xx
r_chrout	sta bank_port0
		jmp newchrout_t		;$3xx
r_getin		sta bank_port0
		jmp newgetin_t		;$3xx
switch_8k	sta cfg_8
		rts
		.fi
;--------------------------------------
		.if BANK=2
		.byte $ff
r_chrin		sta bank_port0
		jmp newchrin_t		;$3xx
r_chrout	sta bank_port0
		jmp newchrout_t		;$3xx
r_getin		sta bank_port0
		jmp newgetin_t		;$3xx
switch_8k	sta cfg_8
		rts
		.fi
;--------------------------------------
		.if BANK=3
		.byte $ff
r_chrin		sta bank_port0
		jmp *-3			;$3xx
r_chrout	sta bank_port0
		sta cfg_open		;$3xx
r_getin		jmp newchrout		;r_getin should not be called from bank3!
		jmp $f142		;$3xx No more raster bugs with getin from keyboard...
switch_8k	sta cfg_8
		rts
		.fi
;--------------------------------------
		.if (DE32_EXISTS=1) & (SUPERCPU=0)
r_keyb		lda ideconf_port
		sta bank_port1
                .if BANK=1
		sta cfg_open
                jmp c128keys
		.fi
		.fi
;-----------------------------------------------------------
; Basic stuff
;
		.if BANK=0
r_mytprt	sta cfg_open
		jmp OP_mylist
r_mydoit	jsr $73
		sta cfg_16
		jsr mydoit		;This already does exit2basic...
		jmp $a7ae
basexeccmd	sta cfg_stnd		;Some optimization for basic...
		jmp $73
basnottoken	sta cfg_stnd
		jsr $b0a1
		jmp $a9a8
		.fi			; BANK=0
;------------------
		.if BANK=3
r_mytprt	sta cfg_open
		jmp OP_mylist
r_mydoit	jsr $73
		sta cfg_16
		jsr mydoit
		jmp $a7ae
basexeccmd	sta cfg_stnd		;Some optimization for basic...
		jmp $73
basnottoken	sta cfg_stnd
		jsr $b0a1
		jmp $a9a8
		.fi			; BANK=3
;------------------

		.here

		.if *>$9ef1
		.error "I/O block $9e60 - $9ef0 overflow!"
		.fi

;--------------------------------------------------------------------
; Kernal extension ($9ef1-)
;

		*= $9ef1
		.logical $def1

write_dt	jsr sysc		;Write    ($def1)
read_dt		jsr sysc		;Read     ($def4)
		.if BANK!=0
sysc		sta bank_port0		;def7
		.else
		.byte $ff,$ff,$ff	;def7
sysc		sta cfg_open		;defa
		jmp syscall		;defd
		.fi			; BANK!=0

		.here

;-----------------------------------------------------------------

		.if *>$9f00
		.error "Kernal extension block $9ef1 - $9eff overflow!"
		.fi

		.if SUPERCPU=0

		*=$9f00
;--------------------------------------
; Handle NMI in opencfg
;
opencfg_nmi	pha			;Soci: only have to save A

		lda #>nmi_return
		pha
		lda #<nmi_return
		pha
		.if nmi_return & %00011000
		.error "BUG, B or D flag set in nmi!"
		.fi
		.if !(nmi_return & %00000100)
		.error "BUG, I flag not set in nmi!"
		.fi
		pha
		jmp nmi_call		;Standard cfg, JMP ($FFFA)
;--------------------------------------
; Handle IRQ in opencfg
;
opencfg_irqbrk
		.if DEBUG=1
		sta help_irqa_exp
		pla
		pha
		and #$10
		bne jump_break		;brk?

		lda #>irq_return
		pha
		lda #<irq_return
		pha
		.if irq_return & %00011000
		.error "BUG, B or D flag set in irq!"
		.fi
		.if !(irq_return & %00000100)
		.error "BUG, I flag not set in irq!"
		.fi
		pha

		lda help_irqa_exp
		jmp irq_call		;Standard cfg, JMP ($FFFE)
		.else			;DEBUG
		pha			;3

		lda #>nmi_return	;5
		pha			;8
		lda #<nmi_return	;10
		pha			;13
		.if nmi_return & %00011000
		.error "BUG, B or D flag set in irq!"
		.fi
		.if !(nmi_return & %00000100)
		.error "BUG, I flag not set in irq!"
		.fi
		pha			;16

		jmp irq_call		;Standard cfg, JMP ($FFFE) 19, 23, 28 cycle delay
		.fi			;DEBUG

;--------------------------------------
; Debugger (C64 screen of death?)
;
		.if DEBUG=1
jump_break	.proc
		cld
		txa
		pha
		lda #$7f
		sta $dd0d
		bit $dd0d
		ldx #0
		stx $d015
		stx $dd02
		lda #27
		sta $d011
		lda #8
		sta $d016
		lda #22
		sta $d018
		lda #2
		sta ident_label_exp
		sta $d021
vw		sta $d800,x
		sta $d900,x
		sta $da00,x
		sta $db00,x
		inx
		bne vw
		ldx #debugtxte-debugtxt
vw2		lda debugtxt-1,x
		sta $428,x
		lda #15
		sta $d828,x
		dex
		bne vw2
		sta bank_port0
		.if BANK=0
		lda help_irqa_exp
		jsr debughex		;ac
		pla
;		jsr debughex		;xr
		jsr debughexy		;yr
		pla
		jsr debughex		;sr
		pla
		sec
		sbc #2
		tay
		pla
		sbc #0
;		jsr debughex
		jsr debughexy
		tsx
		txa
		ldx #2+2+2+2+2+2
		jsr debughex		;sp
		lda $ba
		jsr debughex		;dr
		lda zp_valid_exp
		jsr debughex3
		lda $00
		eor #255
		ora $01
		jsr debughex3		;01
		inx
		ldy #10
vw3		pla
		jsr debughex		;stack
		dey
		bne vw3
		sty $00

		lda #$e2
		sta 792
		lda #$fc
		sta 793
		sta cfg_8
					;Split effekt... (PAL only)
break_loop	sty $d020
		cmp (0,x)
		sta $d020
		nop
		lda #7
		bne break_loop

debughexy	jsr debughex
		tya
debughex	pha
		lsr
		lsr
		lsr
		lsr
		jsr debughex2
		pla
debughex3	and #15
debughex2	cmp #10
		bcc *+4
		adc #6
		adc #$30
		sta $451,x
		lda #1
		sta $d851,x
		inx
		rts
		.fi			; BANK=0

debugtxt	.text "ACXRYRSRPC$",BANK+$30,"SPDRZ1 STACK"
debugtxte
		.pend

		.if *>$a000
		.error "Debugger code overflow $9f00 - $9fff !"
		.fi

		.fi			;DEBUG

		.fi			;SuperCPU
